/*
 *                                 jStock
 *                    (C) Copyright 2003, 2004 Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 26 February 2003, 19:46
 */

package com.exponto.jstock.data;
//import java.io.File;
//import java.io.IOException;
//import java.net.URL;
import java.util.ArrayList;
import java.util.Observable;

import org.apache.log4j.Logger;

import com.exponto.jstock.db.DBIndicatorParameters;
import com.exponto.jstock.db.IndicatorController;
import com.exponto.jstock.db.JstockDB;
import com.exponto.jstock.exception.TickerNotFoundException;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.indicators.IndicatorMaster;
import com.exponto.jstock.indicators.IndicatorParameters;
import com.exponto.jstock.indicators.LogDataSeries;
import com.exponto.jstock.indicators.LogTradingData;
//import com.exponto.jstock.indicators.IndicatorMaster;
import com.exponto.jstock.indicators.IndicatorParametersInterface;
import com.exponto.jstock.io.LoadDataSet;
import com.exponto.jstock.io.TickerID;
import com.exponto.jstock.io.UserPreferences;
import com.exponto.util.Date;
//import com.exponto.jstock.io.ReadNetfondsFile;
//import com.exponto.jstock.io.ReadGenericDataFile;
/**
 * ChartData.java
 *
 * Load the dataset from file or other source. 
 * Can transform dataset to other formats (log, weekly) as required.
 *
 *Does contain some buisness logic (I'll try to move it out of here)
 *
 * @author  kaare
 */
public class ChartData extends Observable {
	static Logger logger = Logger.getLogger(ChartData.class);
	private static final UserPreferences userPrefs = UserPreferences.getInstance();
	private static JstockDB jDB;
	private String m_panelType; // Separates daily from weekly charts
	private TickerID m_tID;
    private LogDataSeries m_logDataSeries;
    private ArrayList m_defaultIndicators; // All supported indicators (from the IndicatorMaster)
    private ArrayList m_dbIndicatorparametersList; 
    private ArrayList m_dbIndicatorList; // Loaded from db
    private ArrayList m_enabledIndicators = new ArrayList(); // Indicators that shall be drawn in separate panel
    private ArrayList m_enabledCandleStickIndicators = new ArrayList(); // // Indicators that shall be drawn together with candelsticks
	private Indicator m_mainIndicator = null;	// THE main indicator (typically the candlestick)

    /**
     * Creates ChartData for a daily series. Should work for weekly as well,
     * but will do the file reading once more (if it is already read for a daily series)
     * 
     * @param tId the TickerID to create ChartData for
     * @param panelType - currently only daily and weekly are supported
     * @throws TickerNotFoundException
     */
    public ChartData(TickerID tId, String panelType) throws TickerNotFoundException {
    	m_panelType = panelType;
		m_tID = tId;
    	int numberOfDays = 1200; // DEFAULT VALUE
    	LoadDataSet loadDataSet = new LoadDataSet();
    	m_logDataSeries = loadDataSet.loadData(m_tID, numberOfDays);
    	if (m_panelType.equals(Constants.weekly)) {
			m_logDataSeries = this.computeWeeklyLogDataSeries(m_logDataSeries);
		}
    	loadEnabledIndicators(m_tID, m_panelType);
    }
   
    /**
     * Used to create ChartData for weekly charts. Uses the the LogDataSeries
     * found for the daily chart. Should work well for daily charts as well if lDS is 
     * null. 
     * 
     * @param tId the TickerID to create ChartData for
     * @param panelType - the type of panel (daily weekly ...) 
     * @param lDS the LogDataSeries from the charts
     * @throws TickerNotFoundException
     */
    public ChartData(TickerID tId, String panelType, LogDataSeries lDS ) throws TickerNotFoundException {
    	m_panelType = panelType;
		m_tID = tId;
		if (lDS == null) {
	    	int numberOfDays = 1200; // DEFAULT VALUE
	    	LoadDataSet loadDataSet = new LoadDataSet();
	    	lDS = loadDataSet.loadData(m_tID, numberOfDays);
		}
		if (m_panelType.equals(Constants.weekly)) {
			m_logDataSeries = this.computeWeeklyLogDataSeries(lDS);
		} else {
			m_logDataSeries = lDS;
		}
		loadEnabledIndicators(m_tID, m_panelType);
    }
    

	/**
	 * Adds new indicators to the chart. 
	 * 
	 * @param indicator - Name of the new indicator
	 */
	public void insertIndicator(ArrayList newIndicators) {
		for (int i = 0; i < newIndicators.size(); i++) {
			Indicator indicator = (Indicator) newIndicators.get(i);
			IndicatorParameters indParam = indicator.getIndicatorParameters();
			indParam.updateParameters(); // Update parameters
			// Creates new DBIndicatorParameters from the indicatorParameters
			DBIndicatorParameters  dbParam = new DBIndicatorParameters(-1, indicator.getIndicatorName(), indParam.toParameterString(), indParam.hashCode(), true);
			DBIndicatorParameters temp = existsInDBIndicatorParameters(dbParam);
			if (temp == null) {
				dbParam.setDisplayed(indicator.getDisplayed());
				// Insert in list if doesn't exist
				this.m_dbIndicatorList.add(indicator);
				this.m_dbIndicatorparametersList.add(dbParam);
				logger.info("New chart specific indicator added: " + indicator);
			} else {
				temp.setDisplayed(indicator.getDisplayed());
				logger.info("Chart specific indicator exists: " + indicator);
			}
		}
	}
	/**
	 * Checks if the IndicatorParameters are known in the database 
	 * before saving them. If new parameters are spotted new 
	 * DBIndicatorParameter are created rather than changing the old since
	 * that may mess up (indicator) lines.
	 *
	 */
	public void saveDBIndicators() {
		for (int i = 0; i < m_dbIndicatorList.size(); i++) {
			Indicator indicator = (Indicator) m_dbIndicatorList.get(i);
			IndicatorParameters indParam = indicator.getIndicatorParameters();
			indParam.updateParameters(); // Update the parameters
			// Creates new DBIndicatorParameters from the indicatorParameters
			DBIndicatorParameters  dbParam = new DBIndicatorParameters(-1, indicator.getIndicatorName(), indParam.toParameterString(), indParam.hashCode(), true);
			DBIndicatorParameters temp = existsInDBIndicatorParameters(dbParam);
			if (temp == null) {
				dbParam.setDisplayed(indicator.getDisplayed());
				// Insert in list if doesn't exist
				this.m_dbIndicatorparametersList.add(dbParam);
				logger.info("IndicatorParameters changed since last save for: " + indicator);
			} else {
				temp.setDisplayed(indicator.getDisplayed());
			}
		}
		jDB.saveIndicatorParameters(m_dbIndicatorparametersList, m_tID, m_panelType);
		findEnabledIndicators(m_dbIndicatorList); // Assumes that the m_dbIndicatorList is always used
		setChanged(); // Inherited from Observable
		notifyObservers(); // Inherited from Observable
	}
	
	/**
	 * Loads the indicators as stored in the database.
	 *
	 */
	public void loadDBIndicators() {
		m_dbIndicatorparametersList = loadIndicatorParameters(m_tID, m_panelType);
		//findEnabledIndicators(m_dbIndicatorList); // Assumes that the m_dbIndicatorList is always used
		//setChanged(); // Inherited from Observable
		//notifyObservers(); // Inherited from Observable
		
	}
	/**
	 * Check if a DBIndicatorParameter already exists
	 * 
	 * @param dbParam - The one to check
	 * @return the DBIndicatorParameter if it exists, null otherwise
	 */
	private DBIndicatorParameters existsInDBIndicatorParameters(DBIndicatorParameters dbParam) {
		DBIndicatorParameters result = null; 
		DBIndicatorParameters temp;
		for (int i = 0; i < m_dbIndicatorparametersList.size(); i++) {
			temp = (DBIndicatorParameters) m_dbIndicatorparametersList.get(i);
			if (dbParam.compareIndicator(temp)) {
				result = temp; // Equal found
			}
		}
		return result;
	}

	/**
	 * Load DBIndicatorParameters from the database
	 * @param tId
	 * @param panelName
	 * @return
	 */
	private ArrayList loadIndicatorParameters(TickerID tId, String panelName) {
		ArrayList indParamList =	jDB.loadIndicatorParameters(tId, panelName);
		return indParamList;
	}
	/**
	 * Gets indicators using the DBIndicatorParameter list
	 *  
	 * @param dbIndicatorParameterList
	 * @return an ArrayList with all indicators that has parameters in the database
	 */
	private ArrayList createIndicatorsFromDB(ArrayList dbIndicatorParameterList) {
		ArrayList indicatorList = new ArrayList();
		DBIndicatorParameters dbIndicatorParameters;
		String indicatorName;
		Indicator indicator;
		for (int i = 0; i < dbIndicatorParameterList.size(); i++) {
			dbIndicatorParameters = (DBIndicatorParameters) dbIndicatorParameterList.get(i);
			if (dbIndicatorParameters.isDisplayed()) {
				indicatorName = dbIndicatorParameters.getIndicatorName();
				indicator = IndicatorMaster.getIndicator(indicatorName); // Must initiate the once from the database
				indicator.initiateIndicator(this.m_logDataSeries, dbIndicatorParameters.getParameterValue());
				indicator.setDisplayed(dbIndicatorParameters.isDisplayed());
				indicatorList.add(indicator);
			} else {
				// TODO: Consider if none visible indicators shall be treated 
				// different, i.e. not initiated at all -> saves time and memory
				// but need to set the parameter string in the indicator, set 
				//logDataSeries to null???
				indicatorName = dbIndicatorParameters.getIndicatorName();
				indicator = IndicatorMaster.getIndicator(indicatorName); // Must initiate the once from the database
				indicator.initiateIndicator(this.m_logDataSeries, dbIndicatorParameters.getParameterValue());
				indicator.setDisplayed(dbIndicatorParameters.isDisplayed());
				indicatorList.add(indicator);
			}
		}
		return indicatorList;
	}
	
    /**
     * Call this only when creating the ChartData class or if the somehow 
     * needs to be refreshed. Loads all indicator related arraylists. If the db is
     * enabled and if some indicator parameters are present, they are used 
     * else the default parameters are used.
     *  
     * @param tId
     * @param panelType
     */
    private void loadEnabledIndicators(TickerID tId, String panelType) {
    	IndicatorController iC = IndicatorController.getInstance();
    	m_defaultIndicators = iC.getIndicators();
    	if (userPrefs.getEnableDB()) {
    		jDB = JstockDB.getInstance();
    		m_dbIndicatorparametersList = this.loadIndicatorParameters(tId, panelType);
    		m_dbIndicatorList = createIndicatorsFromDB(m_dbIndicatorparametersList);
    		if (m_dbIndicatorList.size() == 0) {
    			// Use default indicators
    			this.findEnabledIndicators(m_defaultIndicators);
    		} else {
    			// Use indicators from db
    			this.findEnabledIndicators(m_dbIndicatorList);
    		}
    	} else {
			// Use default indicators if db not supported
    		this.findEnabledIndicators(m_defaultIndicators);
    	}
    	if (m_mainIndicator == null) {
    		// Quick fix to get the mainindicator in case indicators are loaded
    		// from db. The main indicator is not stored in the database.
    		for (int i = 0; i < m_defaultIndicators.size(); i++) {
    			Indicator ind = (Indicator) m_defaultIndicators.get(i);
    			IndicatorParametersInterface indParam = ind.getIndicatorParameters();
    			if (ind.isMainIndicator()) {
    				// Last main indicator in the arraylist is returned (in case there is more than one)
    				m_mainIndicator = ind;
    			}
    		}
    	}
    }
    /**
     * Fills up the ArrayList that contains the enabled
	 * indicators and the enabledCandleStickIndicator list
     *
     *Both arraylist are emptied before they are filled up again.
     */
    private void findEnabledIndicators(ArrayList indicatorList) {
		m_enabledIndicators.clear();
		m_enabledCandleStickIndicators.clear();
		for (int i = 0; i < indicatorList.size(); i++) {
			Indicator ind = (Indicator) indicatorList.get(i);
			IndicatorParametersInterface indParam = ind.getIndicatorParameters();
			if (ind.isMainIndicator()) {
				// Last main indicator in the arraylist is returned (in case there is more than one)
				m_mainIndicator = ind;
			} else 	if (ind.drawWithCandleSticks() && indParam != null) {
				// Check if separate panel is needed
				if (indParam.indicatorEnabled() && ind.getDisplayed()) {
					m_enabledCandleStickIndicators.add(ind);
					//logger.debug("New Indicator without separate panel added to ArrayList");
				}
			} else if (indParam != null) {
				// Check if indicator is enabled
				if (indParam.indicatorEnabled() && ind.getDisplayed()) {
					m_enabledIndicators.add(ind);
					//logger.debug("New Indicator with separate panel added to ArrayList");
				}
			}
		}
		
    }
	
	private void saveIndicatorParameters(ArrayList DBIndParameterList, TickerID tId, String panelType) {
		jDB.saveIndicatorParameters(DBIndParameterList, tId, panelType);
	}

	/**
	 * Computes a weekly LogDataSeries from a daily series
	 * 
	 * @param dailyLDS a Daily loagdataSeries
	 * @return the weekly LogDataSeries 
	 */
	private LogDataSeries computeWeeklyLogDataSeries(LogDataSeries dailyLDS) {
		LogDataSeries weeklyLDS =new LogDataSeries(true);
		double log10 =  Math.log(10);
		LogTradingData tempLTD;
		LogTradingData lTDWeek;
		dailyLDS.setCurrentPeriode();
		tempLTD = dailyLDS.getNextLTD();
		if (tempLTD == null) {
			logger.error("TempLTD is null");
			logger.error(dailyLDS.debugInfo());
		}
		int currentWeek;
		boolean hasMore = true;
		while (hasMore) { 
		//(logDataSeries.hasNextLTD()) {
			lTDWeek = new LogTradingData();
			lTDWeek.low = Constants.lowStart;
			Date tempDate = tempLTD.getDate();
			lTDWeek.setDate(tempDate);
			currentWeek = tempDate.week;
			while (currentWeek == tempDate.week) {
				if (tempLTD.isTraded()) {
					lTDWeek.setStatus(tempLTD.getStatus());
					// Find first valid for open
					if (lTDWeek.open == 0.0) {
						lTDWeek.open = tempLTD.open;
					}
					lTDWeek.volume = lTDWeek.volume + tempLTD.volume;
					// Close is always latest valid value
					lTDWeek.close = tempLTD.close;
					if (tempLTD.low < lTDWeek.low) {
						lTDWeek.low = tempLTD.low;
						lTDWeek.logLow = tempLTD.logLow;
					}
					if (tempLTD.high > lTDWeek.high) {
						lTDWeek.high = tempLTD.high;
						lTDWeek.logHigh = tempLTD.logHigh;
					}
					if (lTDWeek.open > lTDWeek.close) { 
						lTDWeek.logX = Math.log(lTDWeek.open)/log10;
						lTDWeek.logDiff = lTDWeek.logX - Math.log(lTDWeek.close)/log10;
					} else {
						lTDWeek.logX = Math.log(lTDWeek.close)/log10;
						lTDWeek.logDiff = lTDWeek.logX - Math.log(lTDWeek.open)/log10;
					}
				}
				// Only read if there is more data in the series
				if (dailyLDS.hasNextLTD()) {
					tempLTD = dailyLDS.getNextLTD();
					if (tempLTD== null) {
						logger.error("LTD is null, currentWeek is: " + currentWeek);
					}
//					isStored = false;
				} else {
					// Force exit of while loop
					currentWeek = -1;
					hasMore = false;
				} 
			}
			weeklyLDS.storeResultWeekly(lTDWeek);
//			System.out.println("Weekly data - Week no: " + currentWeek +" High: " + lTDWeek.high + " Low: " + lTDWeek.low);			
		}
		return weeklyLDS;
	}

	/**
	 * Returns the number of indicators that needs a separate JPanel in order 
	 * to draw a series. Indicators that are drawn with the candlestick chart 
	 * are not counted nor is indicators that are disabled. 
	 * 
	 * @return the number of panels needed to draw the indicators
	 */
	public int numberOfPanels() {
		return m_enabledIndicators.size();
	}    
	
	/**
	 * 
	 * @return an ArrayList with all Indicators that shall be drawn together 
	 * 				with the candlestickPanel 
	 */
	public ArrayList getEnabledCandleStickIndicators() {
		return m_enabledCandleStickIndicators;
	}
	
	/**
	 * 
	 * @return an ArrayList with all Indicators that shall be drawn in a separate 
	 * 				JPanel
	 */
	public ArrayList getEnabledIndicators() {
		return m_enabledIndicators;
	}
	
/**
 * @return the LogDataSeries 
 */  
	public LogDataSeries getLogDataSeries() {
		return m_logDataSeries;
	}

	/**
	 * @return Returns the TickerID.
	 */
	public TickerID getTickerID() {
		return m_tID;
	}
	
	/**
	 * @return Returns the m_panelName.
	 */
	public String getPanelType() {
		return m_panelType;
	}
	/**
	 * @param name The m_panelName to set.
	 */
	public void setPanelType(String name) {
		m_panelType = name;
	}
	
	/**
	 * @return
	 */
	public Indicator getMainIndicator() {
		return m_mainIndicator;
	}

	/**
	 * @return Returns the m_dbIndicatorList.
	 */
	public ArrayList getAllIndicators() {
		ArrayList list;
		if (m_dbIndicatorList == null) {
			list = m_defaultIndicators;
		} else {
			list = m_dbIndicatorList;
		}
		return list;
	}

	/**
	 * @return Returns the m_defaultIndicators.
	 */
	public ArrayList getDefaultIndicators() {
		return m_defaultIndicators;
	}
}
