/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 18-Mar-2004
 *
 */
package com.exponto.jstock.db;
import com.exponto.jstock.indicators.IndicatorMaster;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.indicators.IndicatorParameters;
import com.exponto.jstock.io.UserPreferences;
//import com.exponto.jstock.indicators.IndicatorParametersInterface;

import java.util.ArrayList;
import javax.swing.JComponent;

import org.apache.log4j.Logger;
/**
 * The responsibility for this class is to handle the communication with the 
 * indicator classes, e.g. to provide new instances of indicator classes.
 * 
 * @author kaare
 *
 */
public class IndicatorController {
	static private IndicatorController instance; // THE instance of this class
	String [] m_indicatorNames; // List of names of all supported indicators
	ArrayList m_indicatorList;
	ArrayList m_supportIndicatorValues;
	ArrayList m_jComponentList;
//	ArrayList enabledIndicators = null;
//	ArrayList enabledCandleStickIndicators = null;
	static Logger logger = Logger.getLogger(IndicatorController.class);
	
	/**
	 * Gets the list of all supported indicators and stores an instance of each in
	 * an ArrayList.
	 */
	private IndicatorController() {
		m_indicatorList = new ArrayList();
		m_jComponentList = new ArrayList();
		m_supportIndicatorValues = new ArrayList();
		m_indicatorNames = IndicatorMaster.getIndicatorsList();
		for (int i = 0; i < m_indicatorNames.length; i++) {
			Indicator ind = IndicatorMaster.getIndicator(m_indicatorNames[i]);
			if (ind.supportIndicatorValuesInterface()) {
				m_supportIndicatorValues.add(ind);
			}
			IndicatorParameters indParam = ind.getIndicatorParameters();
			if (indParam != null) {
				getParametersFromPropertiesFile(indParam);
				JComponent[][] jComp = indParam.getUserInterface();
				indParam.updateParameters();
				m_jComponentList.add(jComp);
				m_indicatorList.add(ind);				
			}

			// 2 lines debug code
//			int [] d  = indParam.getParameters();
//			logger.debug(indicatorNames[i] + " number of days is " + d[0]);
		}
	}

	/**
	 * 
	 * @return THE instance of this class
	 */
	public static IndicatorController getInstance() {
		if (instance == null) {
			instance = new IndicatorController();
		}
		return instance;
	}

	/**
	 * 
	 * @return a String array with the names of the supportred indicators 
	 */
	public String[] getIndicatorNames() {
		return m_indicatorNames;
	}
 	
	public ArrayList getIndicators(ArrayList dbIndicatorParameterList) {
		ArrayList indicatorsList = new ArrayList();
		DBIndicatorParameters dbIndicatorParameters;
		String indicatorName;
		for (int i = 0; i < dbIndicatorParameterList.size(); i++) {
			dbIndicatorParameters = (DBIndicatorParameters) dbIndicatorParameterList.get(i);
			indicatorName = dbIndicatorParameters.getIndicatorName();
			Indicator ind = IndicatorMaster.getIndicator(indicatorName);
			indicatorsList.add(ind);
		}
		return indicatorsList;
	}

/**
 * Gets new instances of every available indicator. This is a little waste of 
 * memory as only enabled indicators will be used. 
 * 
 * @return an ArrayList with new instances of every indicator
 */
	public ArrayList getIndicators() {
		ArrayList indList = new ArrayList();
		for (int i = 0; i < m_indicatorNames.length; i++) {
			Indicator ind = IndicatorMaster.getIndicator(m_indicatorNames[i]);
			indList.add(ind);
		}
		return indList;
	}

	/**
	 * Saves current values of the IndicatorParameters to the properties file.
	 * 
	 * This method is only applicable for default values as chart spesific parameters
	 * will be be saved in db.
	 */
	public void saveAll() {
		for (int i = 0; i < m_indicatorNames.length; i++) {
			Indicator ind = IndicatorMaster.getIndicator(m_indicatorNames[i]);
			IndicatorParameters indParam = ind.getIndicatorParameters();
			if (indParam != null) {
				indParam.updateParameters();
				setParametersToPropertiesFile(indParam);
			}
		}
	}
	/**
	 * Restore all IndicatorParameter default values to whatever is stored in 
	 * the properties file
	 * 
	 * This method is only applicable for default values as chart spesific 
	 * parameters will be be saved in db.
	 */
	public void restoreAll() {
		for (int i = 0; i < m_indicatorNames.length; i++) {
			Indicator ind = IndicatorMaster.getIndicator(m_indicatorNames[i]);
			IndicatorParameters indParam = ind.getIndicatorParameters();
			getParametersFromPropertiesFile(indParam);
		}
	}

	public ArrayList getJComponentList() {
		return m_jComponentList;
	}

	/**
	 * 
	 * @return an ArrayList with indicators that have implemented the 
	 * IndicatorValuesInterface.
	 */
	public ArrayList getIndicatorValues() {
		return m_supportIndicatorValues;
	}
	
	private void getParametersFromPropertiesFile(IndicatorParameters iP) {
		UserPreferences uP =UserPreferences.getInstance();
		String [] parametersNames =iP.getParameterNames();
		for (int i = 0; i < parametersNames.length; i++) {
			int value = -1;
			try {
				value = Integer.parseInt(uP.getProperty(parametersNames[i]));
				//logger.debug("Loading parameters for " + parametersNames[i]);
			} catch (NumberFormatException e) {
				logger.error("Got a NumberFormatException when trying to read " + parametersNames[i]+ " from file, properties file probably corrupt");
			} 
			// Initialize the parameters with default from properties file 
			iP.setParameters(i, value);
		}
	}

	private void setParametersToPropertiesFile(IndicatorParameters iP) {
		UserPreferences uP =UserPreferences.getInstance();
		String [] parametersNames =iP.getParameterNames();
		//iP.setParameters()
		int [] parametersValue = iP.getParameters(); 
		for (int i = 0; i < parametersNames.length; i++) {
				uP.setProperty(parametersNames[i], String.valueOf(parametersValue[i]));
				//logger.debug("Wrote " + parametersNames[i] + " " + String.valueOf(parametersValue[i]) + " to properties file" );             
		}
	}
	
//	/**
//	 * 
//	 * @return an ArrayList with all Indicators that shall be drawn together 
//	 * 				with the candlestickPanel 
//	 */
//		public ArrayList getEnabledCandleStickIndicators() {
//			if (enabledCandleStickIndicators.size() == 0) {
//				// Fills up or checks the arrayList if empty
//				this.numberOfPanels();
//			}
//			return enabledCandleStickIndicators;
//		}
//	/**
//	 * 
//	 * @return an ArrayList with all Indicators that shall be drawn in a separate 
//	 * 				JPanel
//	 */
//		public ArrayList getEnabledIndicators() {
//			if (enabledIndicators.size() == 0) {
//				// Check that this is ok if dis/enabling indicators
//				// Fills up or checks the arrayList if empty
//				this.numberOfPanels();
//			}
//			return enabledIndicators;
//		}
	
//	/**
//	 * Finds the number of indicators that needs a separate JPanel in order 
//	 * to draw a series. Indicators that are drawn with the candlestick chart 
//	 * are not counted nor is indicators that are disabled. 
//	 * 
//	 * Due to laziness this also fills up the ArrayList that contains the enabled
//	 * indicators. Not a very good idea, but someday it might be fixed. 
//	 * 
//	 * @return the number of panels needed to draw the indicators
//	 */
//	public int numberOfPanels() {
//		int number = 0;
//		enabledIndicators.clear();
//		enabledCandleStickIndicators.clear();
//		for (int i = 0; i < indicatorList.size(); i++) {
//			Indicator ind = (Indicator) indicatorList.get(i);
//			IndicatorParametersInterface indParam = ind.getIndicatorParameters();
//			// Check if separate panel is needed
//			if (!ind.drawWithCandleSticks()) {
//				// Check if indicator is enabled
//				if (indParam.indicatorEnabled()) {
//					// A little waste of resource to this everytime, but it works for now
//					enabledIndicators.add(IndicatorMaster.getIndicator(indicatorNames[i]));
//					number ++;
//					//logger.debug("New Indicator with separate panel added to ArrayList");
//				}
//			} else {
//				if (indParam.indicatorEnabled()) {
//					// A little waste of resource to this everytime, but it works for now
//					enabledCandleStickIndicators.add(IndicatorMaster.getIndicator(indicatorNames[i]));
//					//logger.debug("New Indicator without separate panel added to ArrayList");
//				}
//			}
//		}
//		return number;
//	}
}
