/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 09-Sep-2004
 *
 */
package com.exponto.jstock.db;

import java.util.HashMap;
import java.util.Map;

import com.exponto.jstock.io.DBManager;
//import java.sql.*;
import org.apache.log4j.Logger;



/**
 * @author kaare
 * Initialize certain tables in the database. For now all tables must be created 
 * before theese methods are called
 */
public class InitDB {
	static Logger logger = Logger.getLogger(InitDB.class);
	
	/**
	 * Verifies that the database contains all values for certain type of data that
	 * is known by the application. If new ones are found, they are added to 
	 * the database, and false is returned. If the update to the database failed,
	 * an error is logged.
	 * 
	 * The data that can be verified by this method must contain only of a string
	 * field (varchar) and must have a autoincrementing index.
	 * 
	 * This methode ensure that the database will be automatically updated if
	 * e.g. new indicators are added in later versions.  
	 * 
	 * @param map - A hashmap with data loaded from the database
	 * @param table - Name of the database table
	 * @param reference - String [] with the correct values/names
	 * @return true if no new ones are found, false otherwise.
	 */
	public static boolean verifyIndexMap(Map map, String table, String field, String [] reference) {
		boolean resultOK = true;
		if (table == null || reference == null) {
			logger.error("table or refence is null");
			return false;
		}
		if (map == null ) {
			// Create a new empty dummy hashmap
			map = new HashMap();
		}
		String sql;
		int rowCount;
		for (int i = 0; i < reference.length; i ++) {
			if (!map.containsKey(reference[i])) {
				// TODO: Does not deleted indicators if removed from the reference array
				// Probably not needed either as that may cause "unknown" index in e.g. the lines table 
				sql = "INSERT INTO "+ table +" ("+ field + ") VALUES ('" + reference[i] + "')";
				
				rowCount = DBManager.executeUpdate(sql);
				resultOK = false;
				if (rowCount <= 0) {
					logger.error("Problems updating Table " + table + " with " + reference[i]);
					logger.debug(sql);
				}
			}
		}
		return resultOK;
	}
}
