/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 12-Sep-2004
 *
 */
package com.exponto.jstock.db;

import java.net.URL;
import java.util.ArrayList;

import org.apache.log4j.PropertyConfigurator;

import com.exponto.jstock.jStock;
import com.exponto.jstock.gui.panels.PersistentLine;
import com.exponto.jstock.io.TickerID;
import com.exponto.util.Date;

import junit.framework.TestCase;

/**
 * @author kaare
 * 
 * Testcases aren't very well written, but do test most major features.
 *
 */
public class JstockDBTest extends TestCase {
	JstockDB jDB;
	String newTicker = "tada";
	String newExchange = "ose";
	String m_panelType = Constants.panel[1];
	int indicatorId = 2;
	String indicatorName = "RSI";
	TickerID tId;
	private int m_hashCode = 123;
	/**
	 * Constructor for JstockDBTest.
	 * @param arg0
	 */
	public JstockDBTest(String arg0) {
		super(arg0);
		//this.runTest();
	}

	public void runTest() {
		testNewTickerIdNotes();
		testIndicatorParameters();
		testSaveLines();
		testWatchList();
	}

	public void testIndicatorParameters() {
		ArrayList paramList = new ArrayList();
		// Create DBIndicatorParameters
		DBIndicatorParameters dbParam1 = new DBIndicatorParameters();
		dbParam1.setIndicatorName("RSI");
//		dbParam1.setParameterValueNumber(1);
		dbParam1.setParameterValue("14");
		DBIndicatorParameters dbParam2 = new DBIndicatorParameters();
		dbParam2.setIndicatorName("MACD");
//		dbParam2.setParameterValueNumber(1);
		dbParam2.setParameterValue("blablabla");
		paramList.add(dbParam1);
		paramList.add(dbParam2);
		jDB.saveIndicatorParameters(paramList,tId, m_panelType);
		ArrayList paramList2 = jDB.loadIndicatorParameters(tId, m_panelType);
		DBIndicatorParameters dbParam3 = (DBIndicatorParameters) paramList2.get(0);
		DBIndicatorParameters dbParam4 = (DBIndicatorParameters) paramList2.get(1);
		// The index is different so they are copied from the objects read from the db.
		dbParam1.setIndex(dbParam3.getIndex());
		dbParam2.setIndex(dbParam4.getIndex());
		assertTrue(dbParam1.equals(dbParam3));
		assertTrue(dbParam2.equals(dbParam4));
		dbParam4.delete();
		jDB.saveIndicatorParameters(paramList2,tId, m_panelType);
		paramList2 = jDB.loadIndicatorParameters(tId, m_panelType);
		dbParam3 = (DBIndicatorParameters) paramList2.get(0);
		assertTrue(dbParam1.equals(dbParam3));
		assertTrue(paramList2.size()==1);
		dbParam3.delete();
		jDB.saveIndicatorParameters(paramList2,tId, m_panelType);
		paramList2 = jDB.loadIndicatorParameters(tId, m_panelType);
		assertTrue(paramList2.size()==0);
	}
	public void testWatchList() {
		ArrayList watchList = jDB.loadWatchList();
		assertFalse(searchWatchList(watchList, tId));
		jDB.toggleWatchList(tId);
		watchList = jDB.loadWatchList();
		assertTrue(searchWatchList(watchList, tId));
		jDB.toggleWatchList(tId);
		watchList = jDB.loadWatchList();
		assertFalse(searchWatchList(watchList, tId));
		
	}

	/**
	 *  
	 * Loads notes and modifies and verifies the changes
	 * 
	 * Note that "null" must be manually set in db in order to get the
	 * test to pass.
	 * 
	 * @param tId
	 * @return
	 */
	public void testNewTickerIdNotes() {
		String note = "This is a new note";
		String s = jDB.loadNotes(tId);
		assertTrue(s.equals("null"));
		s = jDB.saveNotes(tId, note);
		String str = "Notes saved for ";
		assertTrue(s.startsWith(str));
		s = jDB.loadNotes(tId);
		// Reads notes from database
		System.out.println(s);
		assertTrue(s.equals(note));
		s = jDB.saveNotes(tId, "null");
		assertTrue(s.startsWith(str));
		//return s;
	}
	
	public void testSaveLines() {
		//ArrayList linesArray = this.testSaveLines();
		ArrayList lArray = new ArrayList();
		ArrayList array = new ArrayList();
		int year = 2004;
		int mStart = 0;
		int mStop = 3;
		int dStart = 1;
		int dStop = 31;
		Date start = new Date(year, mStart, dStart);
		Date stop = new Date(year, mStop, dStop);
		double y1 = 12.15;
		double y2 = 23.11;
		PersistentLine pL = new PersistentLine(start, y1, stop, y2);
		lArray.add(pL);
		array.add(pL);
		jDB.saveLines(tId, m_panelType, indicatorName, m_hashCode, array);
		
		ArrayList list = jDB.loadLines(tId, m_panelType, indicatorName, m_hashCode);
		PersistentLine p1, p2;
		p1 = (PersistentLine) list.get(0);
		p2 = (PersistentLine) lArray.get(0);
		//Index will be different as p2 should have the default index of -1
		assertTrue(p2.getLineIndex() == -1);
		p2.setLineIndex( p1.getLineIndex());
		assertTrue(p1.equals(p2));

		PersistentLine pL2 = (PersistentLine) list.get(0);
		pL2.setDeleted(0); 
		lArray = jDB.saveLines(tId, m_panelType, indicatorName, m_hashCode, list);
		assertTrue(lArray.isEmpty());
	}
	
	
	public static void main(String[] args) {		
		junit.textui.TestRunner.run(JstockDBTest.class);
	}

	private boolean searchWatchList(ArrayList wList, TickerID tId) {
		boolean result = false;
		TickerID tId2;
		int index = tId.getTickerIndex();
		for (int i= 0; i< wList.size();i++) {
			tId2 = (TickerID) wList.get(i);
			// Ignore difference in index
			tId.setTickerIndex(tId2.getTickerIndex());
			if (tId.equals(tId2)) {
				result = true;
			}
		}
		// Restore original index
		tId.setTickerIndex(index);
		return result;
	}
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		String resource ="/log.properties";
		//System.out.println("pwd: " + System.getProperty("user.dir"));
		//System.out.println("cp: " + System.getProperty("java.class.path"));
		URL configFileResource = jStock.class.getResource(resource);
		//System.out.println("cp: " + configFileResource.toString());
		PropertyConfigurator.configure(configFileResource);    	
		jDB = JstockDB.getInstance();
		tId = new TickerID(newTicker, newExchange);
		//System.out.println(tId.toDebugString());
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
		jDB = null;
	}


}
