/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 28-Sep-2003
 *
 */
package com.exponto.jstock.gui.chartinfo;
import javax.swing.*;

import java.awt.*;

import javax.swing.table.*;
//import javax.swing.table.AbstractTableModel;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.text.DateFormat;
import org.apache.log4j.Logger;

import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.db.JstockDB;
import com.exponto.jstock.io.TickerID;


/**
 * Singleton for ChartInfo window, used by all "charts", but content is
 * ticker or chart specific.
 * 
 * @author kaare
 *
 */

//TODO : Add global note area (watchlist),  also needs to save notes on quit
// Also add categorys (follow up, buy sell own buy soon)
public class ChartInfo extends JFrame {
	static private ChartInfo instance; // THE instance of this class
	static Logger logger = Logger.getLogger(ChartInfo.class);
	private JTabbedPane tabbedPane;
	private JPanel controlPanel;
	private JPanel mainPanel;
	private JSplitPane mainSplit;
	
	private Notes notes;
	private JScrollPane notesPane;
	private IndicatorsOptions indicatorsOptions;
	private JScrollPane indicatorsOptionsPane;
	private JScrollPane tablePane;
	private JTable table;
	private TickerTableModel tableModel;
	private TableColumn tickerColumn;
	private JComboBox tickerComboBox;
	
	private JButton date;
	private JButton watch;
	private JButton save;
	private JButton restore;
	private JButton clear;
	private JButton close;
	static private JstockDB jDB;
	private TickerID m_currentTID;
	private ArrayList m_indicatorsList;

	
	private ChartInfo() {
		super("Ticker info");
		
//		Components for the text area
		notes = new Notes();
		notesPane = notes.getNotesPane();
		
		indicatorsOptions = new IndicatorsOptions();
		indicatorsOptionsPane = indicatorsOptions.getIndicatorsPane();
		
//		Components for the table area
		
		// Test data
		tableModel = new TickerTableModel();
		table = new JTable(tableModel);
		
		tickerColumn = table.getColumnModel().getColumn(0);
		tickerComboBox = new JComboBox();
		String[] indicators = Rule.getIndicators();
		for (int i = 0; i < indicators.length; i++) {
			tickerComboBox.addItem(indicators[i]);
		}
		tickerComboBox.addItem("Buy (manual override)");
		tickerComboBox.addItem("Hold (manual override)");
		tickerComboBox.addItem("Sell (manual override)");
		tickerComboBox.addItem("None (manual override)");
		//tickerComboBox.setBackground(java.awt.Color.YELLOW);
		tickerColumn.setCellEditor(new DefaultCellEditor(tickerComboBox));
		
		TickerRenderer tickerRenderer = new TickerRenderer();
		tickerRenderer.setToolTipText("Click to adjust buy/sell criteria");
		tickerColumn.setCellRenderer(tickerRenderer);
		
		// Not working
		//table.getComponent(0).setBackground(java.awt.Color.green);
//		Not working don't know why
		//table.setPreferredScrollableViewportSize(new Dimension(500, 70));
		// Uncomment next for extended notes window
		tablePane = new JScrollPane(table);

		tabbedPane = new JTabbedPane();
		tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

		
//		The splitpane
//		Uncomment next for extended notes window
//		mainSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, notesPane, tablePane);
		// Remove next for extended notes window
		mainSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, notesPane, null);
		//mainSplit.add(textPane);
		//mainSplit.add(tablePane);
		mainSplit.setOpaque(true);
		mainSplit.setLeftComponent(notesPane);
		// Uncomment
		//mainSplit.setRightComponent(tablePane);
		mainSplit.setResizeWeight(0.5);
		mainSplit.setOneTouchExpandable(true);
		mainSplit.setContinuousLayout(true);
		//mainPanel = new JPanel();
		//mainPanel.setLayout(new FlowLayout());
		//mainPanel.setOpaque(true);
		//mainPanel.add(mainSplit);
		
		controlPanel();
//		Addes tabbedpane 
		tabbedPane.addTab("Notes", null, mainSplit, null);
		tabbedPane.addTab("Indicators", null, indicatorsOptionsPane, null);
        getContentPane().add(tabbedPane, BorderLayout.CENTER);

		
//		getContentPane().add(mainSplit, BorderLayout.CENTER);
		getContentPane().add(controlPanel, BorderLayout.WEST);
		//getContentPane().setOpaque(true);
		//panel.setLayout(new FlowLayout());
		//scrollPane.add(text); 
		pack();
		setVisible(true);
	}
	
	/**
	 * Gets the one and only instance of this class.
	 * 
	 * @return the only instance of the Notes class.
	 */
	static public ChartInfo getInstance() {
		if (instance==null) {
			instance = new ChartInfo();
		}
		return instance;
	}
	
	/**
	 * Gets the one and only instance of this class, loaded with data for a paritcular 
	 * TickerID. Any previous data that is removed from view is automatically saved 
	 * to the database.  
	 * 
	 * @param ChartData 
	 * @return Notes with data for tID.
	 */
	static public ChartInfo getInstance(ChartData cD) {
		jDB = JstockDB.getInstance();
		if (instance==null) {
			instance = new ChartInfo();
			instance.setCurrent(cD);
		} else {
			instance.save();
			instance.setCurrent(cD);
		}
		instance.clear();
		instance.load();
		return instance;
	}

	/**
	 * Refresh the instance of this class. If the TickerID is equal to the curentTID, 
	 * nothing is done. If not the data is saved to the database and data for the new 
	 * TickerId is loaded.
	 *  
	 * @param tId - TickerID to refresh.
	 */
	public void refresh(ChartData cD) {
		//logger.debug("Notes: Loaded is " + m_currentTID.getTickerName() + " " + m_currentTID.getExchangeName() + " Refreshing " + cD.getTickerID().getTickerName() + " " + cD.getTickerID().getExchangeName());
		if (!(m_currentTID.getTickerName().equalsIgnoreCase(cD.getTickerID().getTickerName())) || !(m_currentTID.getExchangeName().equalsIgnoreCase(cD.getTickerID().getExchangeName()))) {
			instance.save();
			instance.setCurrent(cD);
			indicatorsOptionsPane = indicatorsOptions.getIndicatorsPane();
			instance.load();
			logger.debug(" Refreshing ChartInfo for " + m_currentTID.getTickerName() + " " + m_currentTID.getExchangeName() + " Refreshing " + cD.getTickerID().getTickerName() + " " + cD.getTickerID().getExchangeName());
		}
	}
	
	private void save() {
		notes.save(m_currentTID);
		indicatorsOptions.save();
	}
	
	private void load() {
		notes.load(m_currentTID);
		//indicatorsOptions.restore();
	}
	
	private void clear() {
		notes.clearText();
	}
	
	private void insertDate() {
		String s = instance.getCurrentDate() +"\n";
		notes.insertTextAtTop(s);
	}
	
	private String getCurrentDate() {
		GregorianCalendar cal = new GregorianCalendar();
		java.util.Date calDate = cal.getTime();
		return DateFormat.getDateInstance().format(calDate);
	}
	
//	private void dbConnection() {
//		
//	}
	
	/**
	 * Sets the currentTickerID and refreshes the indicator options.
	 * @param  tID the TickerID
	 */
	public void setCurrent(ChartData cD) {
		m_currentTID = cD.getTickerID();
		m_indicatorsList = cD.getAllIndicators();
		indicatorsOptions.refresh(cD);
		
	}
	
	
//	private void insertText() {
//	}
	/**
	 * Buttons and such for the control panel. 
	 */
	private void controlPanel() {
		controlPanel = new JPanel();
		controlPanel.setLayout(new GridLayout(5, 1));
		date = new JButton("Date");
		date.setMnemonic(KeyEvent.VK_D);
		watch = new JButton("Watch");
		watch.setMnemonic(KeyEvent.VK_W);
		save = new JButton("Apply");
		save.setMnemonic(KeyEvent.VK_S);
		restore = new JButton("Restore");
		restore.setMnemonic(KeyEvent.VK_R);
		clear = new JButton("Clear");
		clear.setMnemonic(KeyEvent.VK_L);
		close = new JButton("Close");
		close.setMnemonic(KeyEvent.VK_Q);
		controlPanel.add(date);
		controlPanel.add(watch);
		controlPanel.add(restore);
		controlPanel.add(clear);
		controlPanel.add(save);
//		controlPanel.add(close);
		date.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				insertDate();
			}
		});
		watch.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				addToWatchList();
			}
		});
		restore.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				load();
			}
		});
		clear.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				clear();
			}
		});
		save.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				save();
			}
		});
		close.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				save();
				instance = null;
				//close();
			}
		});

	}
	
	
	/**
	 * Toggles the whatchlist state of the currentTID. If it's not on the watchlist it is 
	 * added, else it is removed.
	 *
	 */
	private void addToWatchList() {
		if (m_currentTID != null) {
			String str = jDB.toggleWatchList(m_currentTID);
			logger.info("Toggled watchlist for ticker" + str);
		} else {
			logger.error("currentTID is null, don't know what to add");
		}
	}
}
