/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 18-Feb-2004
 *
 */
package com.exponto.jstock.gui.chartinfo;
//import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;

import java.util.ArrayList;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
//import java.awt.event.KeyEvent;

import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.indicators.IndicatorMaster;
import com.exponto.jstock.indicators.IndicatorParameters;
import com.exponto.util.*;
import org.apache.log4j.Logger;
/**
 * @author kaare
 *
 */
public class IndicatorsOptions {
	private JScrollPane m_scrollPane;
	private ArrayList m_allIndicatorsList;
	private ArrayList m_jComponentList;
	private boolean [] m_isDisplayed;
	private ArrayList m_tempIndicatorsList; // Indicators created but not save
	private ChartData m_cD;
	private JPanel mainPanel;
	private JPanel indicatorPanel;
	private JPanel controlPanel;
	//IndicatorController indicatorController;
	static Logger logger = Logger.getLogger(IndicatorsOptions.class);
	private ArrayList m_availableIndicators;
	private static final String position = "position";
	
	public IndicatorsOptions() {
		m_scrollPane = new JScrollPane();
		m_jComponentList = new ArrayList();
		m_tempIndicatorsList = new ArrayList();
		indicatorPanel = new JPanel();
		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
	}
	/**
	 * Buttons and such for the control panel. 
	 */
	private void controlPanel(ArrayList indicators) {
		controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout());
		JLabel add = new JLabel("Add indicator");
		JComboBox indicatorsComboBox = new JComboBox();
		for (int i = 0; i < indicators.size(); i++) {
			Indicator ind = (Indicator) indicators.get(i);
			if (!ind.isMainIndicator()) {
				String name = ind.getIndicatorName();
				indicatorsComboBox.addItem(name);
			}
		}
		//add.setMnemonic(KeyEvent.VK_A);
//		JButton save = new JButton("Save");
//		save.setMnemonic(KeyEvent.VK_S);
//		JButton restore = new JButton("Restore");
//		restore.setMnemonic(KeyEvent.VK_R);

		controlPanel.add(add);
		controlPanel.add(indicatorsComboBox);
		//controlPanel.add(restore);
		//controlPanel.add(save);
		indicatorsComboBox.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				JComboBox cBox = (JComboBox) e.getSource();
				addIndicator((String) cBox.getSelectedItem());
			}
		});
//		restore.addActionListener(new ActionListener () {
//			public void actionPerformed(ActionEvent e) {
//				restore();
//			}
//		});
//		save.addActionListener(new ActionListener () {
//			public void actionPerformed(ActionEvent e) {
//				save();
//			}
//		});
	}

	public void refresh(ChartData cD) {
		m_cD = cD;
		m_jComponentList.clear();
		m_tempIndicatorsList.clear();
		m_allIndicatorsList = cD.getAllIndicators();
		if (m_availableIndicators == null) {
			m_availableIndicators = cD.getDefaultIndicators();
			controlPanel(m_availableIndicators);
			mainPanel.add(controlPanel, BorderLayout.SOUTH);
		}
		if (m_allIndicatorsList != null) {
			initComponents();	
		}
		
	}
	
//	/**
//	 * @param indicators
//	 */
//	private void initControlPanel(ArrayList indicators) {
//		m_controlPanel = controlPanel(indicators);
//		
//	}
	public JScrollPane getIndicatorsPane() {
		mainPanel.remove(indicatorPanel);
		refreshIndicatorPanel();
		indicatorPanel.setOpaque(true);
		mainPanel.add(indicatorPanel, BorderLayout.CENTER);
		m_scrollPane.getViewport().add(mainPanel);
		return m_scrollPane;
	}

	/**
 * 
 */
private void refreshIndicatorPanel() {
	indicatorPanel.removeAll();
	if (m_jComponentList != null) {
		indicatorPanel.setLayout(new GridLayout(m_jComponentList.size(), 2));
		TableLayoutPanel tP;
		for (int i = 0; i < m_jComponentList.size(); i++) {
			JCheckBox checkBox = new JCheckBox("Check to view indicator  ", m_isDisplayed[i]);
			checkBox.putClientProperty(position, new Integer(i));
			checkBox.setHorizontalAlignment(SwingConstants.RIGHT);
			checkBox.setHorizontalTextPosition(SwingConstants.LEFT);
			checkBox.addActionListener(new ActionListener () {
				public void actionPerformed(ActionEvent e) {
					JCheckBox cBox = (JCheckBox) e.getSource();
					Integer index  = (Integer) cBox.getClientProperty(position);
					Indicator ind = (Indicator) m_allIndicatorsList.get(index.intValue());
					ind.setDisplayed(cBox.isSelected());
					//System.out.println("Toggle indicator display status " + ind.getDisplayed());
				}
			});			
			indicatorPanel.add(checkBox);
			tP = new TableLayoutPanel((JComponent [][]) m_jComponentList.get(i));
			tP.setBorder(new LineBorder(Color.BLACK, 1));
			//tP.set
			tP.setOpaque(true);
			indicatorPanel.add(tP);
		}
	} else {
		logger.debug("No indicatorcomponents to display");
	}
}
	public void save() {
		m_cD.insertIndicator(m_tempIndicatorsList);
		m_cD.saveDBIndicators();
		m_tempIndicatorsList.clear();
		this.refresh(m_cD);
//		Indicator ind;
//		IndicatorParameters indParam;
//		// Check if there is existing DBIndicatorParameters for all indicators 
//		for (int i= 0; i< m_allIndicatorsList.size(); i ++) {
//			ind = (Indicator) m_allIndicatorsList.get(i);
//			indParam = ind.getIndicatorParameters();
//			
//		}
	}

	public void restore() {
		m_cD.loadDBIndicators();
		// Reload the panel
		this.refresh(m_cD);
	}
	
	private void addIndicator(String indicator) {
		Indicator ind = IndicatorMaster.getIndicator(indicator);
		// Must initiate the new indicator correctly
		IndicatorParameters indParam = ind.getIndicatorParameters();
		ind.initiateIndicator(m_cD.getLogDataSeries(),indParam.toParameterString());
		m_tempIndicatorsList.add(ind);
		initComponents();
		// 
	}
	
	private void initComponents() {
		m_jComponentList.clear();
		m_isDisplayed = new boolean [m_allIndicatorsList.size()]; 
		for (int i = 0; i < m_allIndicatorsList.size(); i++) {
			// Gets userinterface for existing indicators
			Indicator ind = (Indicator) m_allIndicatorsList.get(i);
			m_isDisplayed[i] = ind.getDisplayed();
			IndicatorParameters indParam = ind.getIndicatorParameters();
			if (indParam != null) {
				//getParametersFromPropertiesFile(indParam);
				JComponent[][] jComp = indParam.getUserInterface();
				indParam.updateParameters();
				m_jComponentList.add(jComp);
			}
		}
		for (int i = 0; i < m_tempIndicatorsList.size(); i++) {
			// Gets userinterface for new indicators			
			Indicator ind = (Indicator) m_tempIndicatorsList.get(i);
			IndicatorParameters indParam = ind.getIndicatorParameters();
			if (indParam != null) {
				//getParametersFromPropertiesFile(indParam);
				JComponent[][] jComp = indParam.getUserInterface();
				indParam.updateParameters();
				m_jComponentList.add(jComp);
			}
		}
		refreshIndicatorPanel();
	}

}
