/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 19-Nov-2004
 * 
 */
package com.exponto.jstock.gui.chartinfo;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.apache.log4j.Logger;

import com.exponto.jstock.db.JstockDB;
import com.exponto.jstock.io.TickerID;

/**
 * Implements a editable textarea in a JScrollpane that may be used for 
 * taking notes.
 * 
 * Maybe used both in a "dynamic" way (new text for each ticker) or a 
 * "static" available for all tickers 
 * 
 * @author kaare
 *
 */
public class Notes {
	private JScrollPane textPane;
	private JPanel textPanel;
	private JTextArea text;
	static private JstockDB jDB;
	static Logger logger = Logger.getLogger(Notes.class);
	
	/**
	 * 
	 */
	public Notes() {
		
		// Components for the textarea 
		textPane = new JScrollPane();
		textPanel = new JPanel();
		text = new JTextArea("\nAdd your own comments here\n");
		textPanel.add(textPane, BorderLayout.CENTER);
		text.setWrapStyleWord(true);
		text.setEditable(true);
		text.setColumns(25);
		text.setRows(10);
		textPane.getViewport().add(text);

	}

	
	public JScrollPane getNotesPane() {
		return textPane;
	}
	/**
	 * Save text in the notes window to the database.
	 *
	 */
	public void save(TickerID tID) {
		String s;
		this.text.selectAll();
		String str = this.text.getSelectedText();
		if (str != null && tID != null) {
			jDB = JstockDB.getInstance();
			s = jDB.saveNotes(tID, str);
//			logger.debug("Notes - tried saving");
		} else {
			logger.info("No text to save");
		}
		this.text.moveCaretPosition(0);
	}

	/**
	 * Loads text for the notes window from the database.
	 *
	 */
	public void load(TickerID tID) {
		jDB = JstockDB.getInstance();
		String str = jDB.loadNotes(tID);
		//System.out.println("str");
		if (str == null || str.equals(" ")) {
			this.text.replaceSelection("\nAdd your own comments here");
		}
		this.text.selectAll();
		this.text.replaceSelection(str);
	}

	
	public void insertTextAtTop(String s) {
		this.text.selectAll();
		String str = this.text.getSelectedText();
		
		this.text.replaceSelection(s + str);
		this.text.moveCaretPosition(str.length());
//		System.out.println(instance.getCurrentDate());
	}
	
	public void clearText() {
		this.text.selectAll();
		this.text.replaceSelection("");
	}


	
}
