/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 18-Feb-2004
 *
 */
package com.exponto.jstock.gui.options;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import com.exponto.jstock.io.UserPreferences;
import com.exponto.util.*;
/**
 * Generates the content for the Database tab in the Options menu.
 * 
 * @author kaare
 *
 */
public class DatabaseMenu {
	JComponent [][] compArray;
	private JLabel enableDB;
	private JCheckBox enableDBBox;
	private JLabel jdbcDriver;
	private JTextField jdbcDriverInput;
	private JLabel dbName;
	private JTextField dbNameInput;
	private JLabel dbHost;
	private JTextField dbHostInput;
	private JLabel dbUser;
	private JTextField dbUserInput;
	private JLabel dbPW;
	private JPasswordField dbPWInput;
	
	private TableLayoutPanel tP;
	private JPanel dbPanel;
	private JPanel emptyPanel; 
	
	int rows = 5;
	int columns = 2;
	UserPreferences userPref;
//	static Dimension threeDigitDim = new Dimension(36, 14);	
	
	public DatabaseMenu() {
		//System.out.println("Jlabel: " + rsi.toString());
		//System.out.println("JTextField: " + rsiText.toString());
		initComponents();
		initCompArray();	
	}

	public JPanel getPanel() {
		dbPanel = new JPanel();
		emptyPanel = new JPanel();
		dbPanel.setLayout(new BorderLayout());
		JPanel checkBoxPanel = new JPanel();
		checkBoxPanel.setLayout(new FlowLayout());
		checkBoxPanel.add(enableDB);
		checkBoxPanel.add(enableDBBox);
		dbPanel.add(checkBoxPanel, BorderLayout.NORTH);
		tP = new TableLayoutPanel(compArray);
		tP.setOpaque(true);
		dbPanel.setOpaque(true);
		emptyPanel.setOpaque(true);
		enableDBBox.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				userPref.setEnableDB(!userPref.getEnableDB());
				if (userPref.getEnableDB()) {
					dbPanel.remove(emptyPanel);
					dbPanel.add(tP, BorderLayout.CENTER);	
				} else {
					dbPanel.remove(tP);
					dbPanel.add(emptyPanel);
				}
				// Redraws the panel.
				//dbPanel.revalidate();
				dbPanel.repaint();
				//System.out.println(" Trykker p� knappen og n� er den " + userPref.getEnablePos());
//				setActiveInputParameters();
			}
		});
		
		if (userPref.getEnableDB()) {
			dbPanel.add(tP, BorderLayout.CENTER);	
		}
		dbPanel.revalidate();
		return dbPanel;
	}
	public void save() {
		userPref.setJdbcDriver(jdbcDriverInput.getText());
		userPref.setDbHost(dbHostInput.getText());
		userPref.setDbName(dbNameInput.getText());
		userPref.setDbUser(dbUserInput.getText());
		userPref.setDbPW(dbPWInput.getText());
	}
	public void restore() {
		
		jdbcDriverInput.setText(userPref.getJdbcDriver());
		dbHostInput.setText(userPref.getDbHost());
		dbNameInput.setText(userPref.getDbName());
		dbUserInput.setText(userPref.getDbUser());
		dbPWInput.setText(userPref.getDbPW());
	}
	
	private void initComponents() {
		userPref = UserPreferences.getInstance();
		
		enableDB = new JLabel("Use db");
		enableDBBox = new JCheckBox("Check to enable", userPref.getEnableDB());
		jdbcDriver = new JLabel("JDBC driver");
		jdbcDriverInput = new JTextField(userPref.getJdbcDriver(), 30);
		//System.out.println(jdbcDriverInput.toString());
		dbHost = new JLabel("DB Host");
		dbHostInput = new JTextField(userPref.getDbHost(), 30);
		dbName = new JLabel("DB Name");
		dbNameInput = new JTextField(userPref.getDbName(), 10);
		dbUser = new JLabel("DB User");
		dbUserInput = new JTextField(userPref.getDbUser(), 10);
		dbPW = new JLabel("DB Password");
		dbPWInput = new JPasswordField(userPref.getDbPW(), 10);

		
		String dbHostTip = "Enter the hostname of your database server. If the database is running on this machine 'localhost' should work. The hostname can either be an ip adress or a known hostname.";
		String dbTip = "Enter the password for the database user. Note that the password will be stored in clear text";
//		String maTip = "Enter the number of days to use in moving average calculation. \nUse 0 for no moving average.";
		
		//
		this.restore();
//		rsiText.setToolTipText(rsiTip);
//		rsi.setToolTipText(rsiTip);

		dbHost.setToolTipText(dbHostTip);
		dbHostInput.setToolTipText(dbHostTip);
		
		dbPW.setToolTipText(dbTip);
		dbPWInput.setToolTipText(dbTip);
		//maText.setPreferredSize(threeDigitDim);
//		rsiText.setPreferredSize(threeDigitDim);
	}
	
	private void initCompArray() {
		compArray = new JComponent[rows][];
		for (int i = 0; i < rows; i ++) {
			compArray[i] = new JComponent[columns]; 
		}
		compArray[0][0] = jdbcDriver;
		compArray[0][1] = jdbcDriverInput;
		compArray[1][0] = dbHost;
		compArray[1][1] = dbHostInput;
		compArray[2][0] = dbName;
		compArray[2][1] = dbNameInput;
		compArray[3][0] = dbUser;
		compArray[3][1] = dbUserInput;
		compArray[4][0] = dbPW;
		compArray[4][1] = dbPWInput;
		
	}
}
