/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * ChartPanel.java
 *
 * Created on 06 December 2001, 22:02
 */

package com.exponto.jstock.gui.panels;

import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;
import java.awt.event.KeyEvent;
import org.apache.log4j.Logger;
import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.gui.chartinfo.ChartInfo;
import com.exponto.jstock.gui.panels.KeyListener;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.indicators.LogDataSeries;
import com.exponto.jstock.io.UserPreferences;

/**
 * Responsibility of this class is to decide what panels are needed and to calculate their size and
 * posistion. Also loads the intial dataset, but further processing of that may be required by the 
 * "drawing" classes.
 *
 *
 * @author  kaare
 * @version 
 */

public class ChartPanel extends JPanel implements Observer{
	private static Logger logger = Logger.getLogger(ChartPanel.class);
	private static UserPreferences userPrefs = UserPreferences.getInstance();
	private static ChartInfo sm_notes;   
    private static Dimension sm_currentDim = new Dimension();
    private Dimension m_indicatorPanelDim = new Dimension();
    private Dimension m_candleDim= new Dimension();
	private  ArrayList m_indicatorList;
//    private CandleStickPanel m_candleStickPanel;
	private AbstractIndicatorPanel m_mainIndicatorPanel;
    private GraphicsParameters m_graphicsParameters = new GraphicsParameters();
    private GraphicsParameters m_mainGraphicsParameters = new GraphicsParameters();
    //private DisplayWindow m_displayWindow;
    private ChartData m_cD;
	private JPanel [] m_indicatorPanelsArray;
	
	// Key and mouse input
	int m_selctedPanel = -1; // The number of the panel that currently has the focus 
	int m_SelectedStick = -1; // Points to selected stick
	int m_yMouseOffset = 0; // Contains the offset in the y direction from top of this panel to start of the m_currentIndicatorPanel
	boolean m_lineSelected = false; // Used for control of arrow keys (for lines or selected stick).
	private AbstractIndicatorPanel m_currentIndicatorPanel;
	
	/**
	 * 
	 *
	 */
	public ChartPanel() {
		 JLabel dummy = new JLabel("Loading the first chart");
		 this.setLayout(new FlowLayout());
		 this.add(dummy);
		 //pack();
		setVisible(true);
		repaint();
		sm_currentDim.setSize(this.getWidth(), this.getHeight());
		System.out.println("cP size: " + sm_currentDim.toString());
	}
	
    /** 
     * Creates new ChartPanel 
     * 
     */
    public ChartPanel(ChartData chartData)  {
    	//this.addComponentListener(new ChartPanelSizeChangedListener());
        m_cD = chartData;
        m_cD.addObserver(this);
        this.setSize(sm_currentDim);
        //m_displayWindow = new DisplayWindow();
        //m_candleStickPanel = new CandleStickPanel(m_cD);
        makePanels();
		setVisible(true);  
		//System.out.println("ChartPanel done, size is: " + sm_currentDim.toString());
	}
	
    public static void setPanelSize(Dimension dim) {
    	sm_currentDim = dim;
    }
    
	public ChartData getChartData() {
		return m_cD;
	}

	private void makePanels() {
		LogDataSeries lDS;
		lDS = m_cD.getLogDataSeries();
		m_indicatorList = m_cD.getEnabledIndicators();
		updatePanelsSize();
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		//m_candleStickPanel = new CandleStickPanel(m_cD, m_mainGraphicsParameters);
		m_mainIndicatorPanel = new MainIndicatorPanel(m_cD, m_cD.getMainIndicator());
		m_mainIndicatorPanel.setGraphicsParameters(m_mainGraphicsParameters);
		mouseInput();
		keyInput();
		this.add(m_mainIndicatorPanel);
		m_indicatorPanelsArray = new IndicatorPanel [m_indicatorList.size()];
		for (int i = 0; i < m_indicatorList.size(); i++) {
			Indicator ind = (Indicator) m_indicatorList.get(i);
			
//			Create separate IndicatorPanel for graphs
			IndicatorPanel indicatorPanel = new IndicatorPanel(m_cD, ind);
			m_indicatorPanelsArray[i] = indicatorPanel;
			indicatorPanel.setGraphicsParameters(m_graphicsParameters);
			this.add(indicatorPanel);
//			logger.debug("Added Indicator panel number: " + i);
		}
			//logger.info("Class ChartPanel, rsiPanel not null and added to layoutmanager");
		if (userPrefs.getNotes()) {
			if (sm_notes == null) {
				sm_notes = ChartInfo.getInstance(m_cD);
			} else {
				sm_notes.refresh(m_cD);
			}
		}
		// Attempt to fix initial size error (not working)
		//m_mainIndicatorPanel.repaint();
	}
 /**
  * Recalculates all panel sizes used by this chart panel
  *
  */
	public void updatePanelsSize() {
		double scaling;
		int minChartHeight = 40;
		int defaultIndicatorPanelHeight = 15;
		sm_currentDim.setSize(this.getWidth(), this.getHeight());
		if (this.getWidth() == 0 && this.getHeight() == 0) {
			sm_currentDim.setSize(200,200);
		}
		String csHeight = userPrefs.getProperty(Constants.minCSHeight);
		String iPHeight = userPrefs.getProperty(Constants.DefaultIndicatorHeight);
		if (csHeight == null) {
			userPrefs.setProperty(Constants.minCSHeight, Integer.toString(minChartHeight));
		} else {
			try {
				minChartHeight = Integer.parseInt(csHeight) ;            
			} catch (NumberFormatException e) {
				logger.error("Got a NumberFormatException when trying to read Scaling.MinimumCandlestickHeight from file, properties file probably corrupt");
			}
		}
		if (iPHeight == null) {
			userPrefs.setProperty(Constants.DefaultIndicatorHeight, Integer.toString(defaultIndicatorPanelHeight));
		} else {
			try {
				defaultIndicatorPanelHeight = Integer.parseInt(iPHeight) ;            
			} catch (NumberFormatException e) {
				logger.error("Got a NumberFormatException when trying to read Scaling.DefaultIndicatorPanelHeight from file, properties file probably corrupt");
			}
		}
		int numberOfPanels = m_cD.numberOfPanels();
		//logger.debug("Number of panels is " + numberOfPanels);
		if (numberOfPanels < 1) {
			scaling = 0;
		} else if ((100 - numberOfPanels * defaultIndicatorPanelHeight) < minChartHeight ) {
			scaling = (double) (100 - minChartHeight) / numberOfPanels;
		} else {
			scaling = (double) numberOfPanels * defaultIndicatorPanelHeight/100;
		}
		m_indicatorPanelDim.setSize(sm_currentDim.getWidth(), (sm_currentDim.getHeight()) * (scaling) / numberOfPanels);
		m_candleDim.setSize(sm_currentDim.getWidth(), (sm_currentDim.getHeight())  * (1 - scaling));
//		logger.debug("CurrentDim: " + currentDim.toString());
//		logger.debug("CandleDim: " + candleDim.toString());
//		logger.debug("indicatorPanelDim: " + indicatorPanelDim.toString());
		int start =(int) (sm_currentDim.getWidth() - userPrefs.getLMargin() - userPrefs.getRMargin())/ userPrefs.getPixelPrStick();
		// Sets the relative end of the displayed periode to 0, i.e. the last day in the series
		int end = 0;
		m_graphicsParameters.setRelativeEndOfPeriode(end);
		m_graphicsParameters.setDimension(m_indicatorPanelDim);
		m_mainGraphicsParameters.setRelativeEndOfPeriode(end);
		m_mainGraphicsParameters.setDimension(m_candleDim);
		m_mainGraphicsParameters.setTopMargin(userPrefs.getTMargin());
		m_mainGraphicsParameters.setBottomMargin(userPrefs.getBMargin());
		//System.out.println("static cP dim:" + sm_currentDim.toString() );
	}
	
	private void keyInput()  {
		KeyListener kL = new KeyListener() {
		
			public void keyPressed(KeyEvent e) {
				//char ch = e.getKeyChar();
				int keyCode = e.getKeyCode();
				if (userPrefs.getDrawMode() && m_lineSelected) {
			    	// Move line. In x direction movment is limited to about the candlesticksize due
			    	// to that the date is used for persitence
			    	int movePixel = 1;
					if (keyCode == KeyEvent.VK_RIGHT) {
						moveLine(userPrefs.getPixelPrStick(), 0);
					} else if (keyCode == KeyEvent.VK_LEFT) {
						moveLine(-userPrefs.getPixelPrStick(), 0);
					} else if (keyCode == KeyEvent.VK_UP) {
						moveLine(0, -movePixel);
					} else if (keyCode == KeyEvent.VK_DOWN) {
						moveLine(0, movePixel);
					}
				} else {
					// Change selected day
					if (keyCode == KeyEvent.VK_RIGHT) {
						moveSelectedStick(1);
					} else if (keyCode == KeyEvent.VK_LEFT) {
						moveSelectedStick(-1);
					}
				}
			}

			public void keyReleased(KeyEvent e) {
				//candleStick.keyReleased(e.getKeyCode());
	//			System.out.println(" Key released");
			}
	
			/* (non-Javadoc)
			 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
			 */
			public void keyTyped(KeyEvent e) {
				//candleStick.keyTyped(e.getKeyChar());
				char ch = e.getKeyChar();
				if (ch == 'n') {
//					Next
					moveSelectedStick(1);
				} else if (ch == 'p') {
					// Previous
					moveSelectedStick(-1);
				} else if (ch == 'v') {
					toggleViewMode();
				} else if (ch == 'l') {
					selectNextLine(true);
				} else if (ch == 'L') {
					selectNextLine(false);
				} else if (ch == '\u007f' || ch ==  'd') {
					//\u007f is delete key
					deleteSelectedLine();
				} else if (ch ==  '\u0013') {
					// \u0013 is ctrl s
					saveLines();
				}
			}
		};
		this.addKeyListener(kL);
		//m_mainIndicatorPanel.addKeyListener(kL);
		add(m_mainIndicatorPanel);
		setVisible(true);
		m_mainIndicatorPanel.repaint();    	
	}
  
    /**
	 * @param number - Number of days to move the selcted stick
	 */
	protected void moveSelectedStick(int number) {
		m_SelectedStick = m_SelectedStick + number;
		m_mainIndicatorPanel.updateDailyPrintout(m_SelectedStick);
		for (int i = 0; i < m_indicatorPanelsArray.length; i ++) {
			IndicatorPanel iP = (IndicatorPanel) m_indicatorPanelsArray[i];
			iP.updateDailyPrintout(m_SelectedStick);
			iP.repaint();
		}
	}

    /**
	 * @param value
	 */
	protected void setSelectedStick(int value) {
		m_SelectedStick = value;
		m_mainIndicatorPanel.updateDailyPrintout(m_SelectedStick);
		for (int i = 0; i < m_indicatorPanelsArray.length; i ++) {
			IndicatorPanel iP = (IndicatorPanel) m_indicatorPanelsArray[i];
			iP.updateDailyPrintout(m_SelectedStick);
			iP.repaint();
		}
	}
	
	/**
	 * Returns the indicatorpanel where the mouse pointer is currently located
	 * 
	 * @return the current  indicatorpanel
	 */
	private AbstractIndicatorPanel getCurrentPanel() {
		return m_currentIndicatorPanel;
	}

	private void mouseInput() {
		MouseInputListener mil = new MouseInputListener() {
			public void mousePressed(MouseEvent e) {
				if (e.getButton() ==  Constants.rightButton) {
					startNewLine(e.getX(), e.getY());
					//System.out.println("mousePressed - Start new line");
				}
			}
			public void mouseDragged(MouseEvent e) {
				// Right Button down
				// Check if button 3 (right) is down
				if (e.getModifiers() ==  java.awt.event.InputEvent.BUTTON3_MASK) {
					drawLine(e.getX(), e.getY());
					//System.out.println("mouseDragged - Draw temp line");
				}
			}
			public void mouseMoved(MouseEvent e) {
				int i = findSelectedPanel(e.getY());
				// No button down
				//System.out.println("mouseMoved");
			}
			public void mouseClicked(MouseEvent e) {
				// left button clicked
				if (e.getButton() == Constants.leftButton)  {
					setSelectedStick((e.getX() - userPrefs.getLMargin()) /userPrefs.getPixelPrStick());
				}
			}
			public void mouseReleased(MouseEvent e) {
				if (e.getButton() ==  Constants.rightButton) {
					endNewLine(e.getX(), e.getY());
				}
			}
			public void mouseEntered(MouseEvent e) {
				requestFocusInWindow();
				//System.out.println("mouseEntered, focus requsted");
			}
			public void mouseExited(MouseEvent e) {
				//System.out.println("mouseExited");
			}
		};
		this.addMouseListener(mil);
		this.addMouseMotionListener(mil);
    }

	/**
     * Position for the first point in a new line
	 * @param x
	 * @param y
	 */
	protected void startNewLine(int x, int y) {
		// Draw line mode
		if (userPrefs.getDrawMode()) {
			AbstractIndicatorPanel iP = getCurrentPanel();
			iP.startIndicatorLine(x, y - m_yMouseOffset);
		}
	}

	/**
	 * Temporary position of the end point of a new line (used while dragging the mouse)
	 * 
	 * @param x
	 * @param y
	 */
	protected void drawLine(int x, int y) {
		if (userPrefs.getDrawMode()) {
			AbstractIndicatorPanel iP = getCurrentPanel();
			iP.drawTemporaryLine(x, y - m_yMouseOffset);
		}
	}

	/**
	 * End point of the new line
	 * @param x
	 * @param y
	 */
	protected void endNewLine(int x, int y) {
		if (userPrefs.getDrawMode()) {
			AbstractIndicatorPanel iP = getCurrentPanel();
			iP.endIndicatorLine(x, y - m_yMouseOffset);
		}
	}
	
	/**
	 * Moves a selected line (x,y) pixels
	 * 
	 * @param x - Number of pixels to move in x direction
	 * @param y - Number of pixels to move in y direction
	 */
	protected void moveLine(int x, int y) {
		AbstractIndicatorPanel iP = getCurrentPanel();
		iP.moveLine(x, y);
		iP.repaint();
	}

	/**
	 * Saves all lines for all panels to the database 
	 */
	protected void saveLines() {
		// Save lines for all panels even if view is off 
		m_mainIndicatorPanel.save();
		for (int i = 0; i < m_indicatorPanelsArray.length; i ++) {
			IndicatorPanel iP = (IndicatorPanel) m_indicatorPanelsArray[i];
			iP.save();
		}
	}

	/**
	 * Delete the selected line in the current indicator panel
	 * Does currently not delete lines in the other indicator panels.
	 * 
	 * By letting deleteSelectedLine return the number of lines left, it could 
	 * be easy to go to another panel if desired.
	 */
	protected void deleteSelectedLine() {
		// Must only deleted a line if they are displayed and one is selcted
		if (userPrefs.getDrawMode()) {
			AbstractIndicatorPanel iP = getCurrentPanel();
			iP.deleteSelectedLine();
			iP.repaint();
		}
	}
	/**
	 * Selects the next line in an indicator panel. 
	 *
	 * @param selectNewLine if true a new line is selcted, otherwise the next 
	 * endpoint is selected
	 */
	protected void selectNextLine(boolean selectNewLine) {
		if (userPrefs.getDrawMode()) {
			AbstractIndicatorPanel iP = getCurrentPanel();
			m_lineSelected = iP.selectNextLine(selectNewLine);
			iP.repaint();
		}
	}

	/**
	 * Turns on or off view of lines for all indicators
	 */
	protected void toggleViewMode() {
		// Toggles drawmode in userpreferences
		userPrefs.setDrawMode(!userPrefs.getDrawMode());
	}


	public void refresh() {
		//System.out.println("cP size: " + sm_currentDim.toString());
		if (userPrefs.getNotes()) {
			if (sm_notes == null) {
				sm_notes = ChartInfo.getInstance(m_cD);
			} else {
				sm_notes.refresh(m_cD);
			}
		}  
		
    }
    
	public void paint(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		super.paint(g2d);
		// Must check if the indicator lists in ChartData has changed
		
	}

	public static void closingTime() {
		
	}
	/**
	 * Finds the indicator panel that has been clicked by the mouse. marks the 
	 * found panel as the current indicator panel.
	 * @param yGlobal
	 * @return the y postion in the current (selected) panel
	 */
	private int findSelectedPanel(int yGlobal) {
		int value = -1;
		//System.out.println("y value, candle height: " + yGlobal + ", " + m_candleDim.getHeight());
		if (m_candleDim.getHeight() >= yGlobal ) {
			// CandleStickPanel clicked 
			m_currentIndicatorPanel = m_mainIndicatorPanel;
			value = yGlobal;
			m_yMouseOffset = 0;
			//System.out.println("Focus requested for main");
		} else {
			int y = yGlobal - (int) m_candleDim.getHeight();
			m_yMouseOffset = (int) m_candleDim.getHeight();
			int i;
			for (i = 1; y >= i * m_indicatorPanelDim.getHeight(); i++) {
				m_yMouseOffset = m_yMouseOffset + (int) m_indicatorPanelDim.getHeight();
				//System.out.println("Next panel " +  i * m_indicatorPanelDim.getHeight());
			}
			if (m_indicatorPanelsArray.length >= (i -1)) {
				m_currentIndicatorPanel = (IndicatorPanel) m_indicatorPanelsArray[i-1];
				value = yGlobal - m_yMouseOffset;
				//System.out.println("Focus requested for other: " + (i-1));
			}
		}
		return value;
	}

	/* (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable o, Object arg) {
		if (o  instanceof ChartData) {
			this.removeAll();
			m_mainIndicatorPanel.removeAll();
			m_indicatorPanelsArray = null;
			this.makePanels();
			this.repaint();
			logger.debug("ChartData changed for " + m_cD.getTickerID().toString());
		}
	}
	
//	private void updateIndicatorStatus(JPanel [] indList, int number) {
//		for (int i = 0; i < indList.length; i++) {
//			IndicatorPanel indPanel = (IndicatorPanel) indList[i];
//			indPanel.updateDailyPrintout(number);
//		}
//	}

//	public static void setAvailableSize(Dimension dim) {
//	//Does not correct the size if more than one rows of tabs are added.
//	// see initComponents() in jStock for further comments on tabs. 
//	currentDim.setSize(dim.getWidth() - 5, dim.getHeight() - 27);
//	logger.debug("currentDim set by jStock: "+ currentDim.toString());
//	//updatePanelsSize();
//	
//}	
//	public void panelRequestFocus() {
//		boolean focus = requestFocusInWindow(false);
//		//System.out.println("mouseEntered, focus is " + focus);
//		System.out.println("Focus  " + focus);
//	}
	
//		g2d.drawString(" Just a test of drawing from the super class as well, 1, 2, 3 testing ",  100, 100);
// This work good as gold, may use this to move (most of) the mouse part out of the candlestick class
//	private void makeWeeklyPanel() {
//	indicatorList = updatePanelsSize();
//	graphicsParameters.setDimension(indicatorPanelDim);
//	this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
////	This one gets a complete LogDataSeries
//	candleStick = new CandleStickPanel(cD.getWeeklyLogDataSeries());
//	candleStick.setPreferredSize(candleDim);
//	mouseInput();
//	keyInput();
//	this.add(candleStick);
//
//	for (int i = 0; i < indicatorList.size(); i++) {
//		Indicator ind = (Indicator) indicatorList.get(i);
////		Create separate IndicatorPanel for graphs
//		IndicatorPanel indicatorPanel = new IndicatorPanel(cD.getWeeklyLogDataSeries(), ind);
//		indicatorPanel.setGraphicsParameters(graphicsParameters);
//		this.add(indicatorPanel);
////		logger.debug("Added Indicator panel number: " + i);
//	}
//		//logger.info("Class ChartPanel, rsiPanel not null and added to layoutmanager");
//	if (userPrefs.getNotes()) {
//		if (notes == null) {
//			notes = Notes.getInstance(cD.getTickerId());
//		} else {
//			notes.refresh(cD.getTickerId());
//		}
//	}
//}
}
