/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 11-Mar-2004
 *
 */
package com.exponto.jstock.gui.panels;
import java.awt.Dimension;
import org.apache.log4j.Logger;

import com.exponto.jstock.io.UserPreferences;

/**
 * This class encapsulates a few parameters that are needed 
 * in order to draw the indicator correctly on screen. 
 * 
 * @author kaare
 *
 */
public class GraphicsParameters implements Cloneable {
	UserPreferences uP;
	private Dimension dim;
	private double logaritmicMinimumValue;
	private double logaritmicMaximumValue;
	private int topMargin; // No need to use this for separate panels
	private int bottomMargin; // No need to use this for separate panels
	private int leftMargin;
	private int rightMargin;
	//private int pixelPrDay;
	
	static Logger logger = Logger.getLogger(GraphicsParameters.class);
	private int m_relativeEndOfPeriode;
	
	public GraphicsParameters() {
	}

	public Object clone() {
		GraphicsParameters o = null;
        try {
        	o = (GraphicsParameters) super.clone();
        } catch(CloneNotSupportedException e) {
        	logger.error(e.toString());
        }
        o.dim = (Dimension) o.dim.clone();
        return o;
        
//		GraphicsParameters clone = new GraphicsParameters();
//		clone.setDimension((Dimension) this.getDimension().clone());
//		clone.setLogaritmicMaximumValue(this.getLogaritmicMaximumValue());
//		clone.setLogaritmicMinimumValue(this.getLogaritmicMinimumValue());
//		clone.setTopMargin(this.getTopMargin());
//		clone.setBottomMargin(this.getBottomMargin());
//		clone.setState();
//		return clone;
	}
	/**
	 * Compares a few size related parameters to determine if oldGP is
	 * equal to this GraphicsParameters. Note that not all parameters
	 * are checked, and if this method is used beyond intentions it may 
	 * give false positive
	 *  
	 * @param oldGP
	 * @return true if they are considered equal (see note above), false 
	 * otherwise
	 */
	public boolean isEqual(GraphicsParameters oldGP) {
		boolean equal = true;
		if (this.stateChanged(oldGP)){
			equal = false;
//			logger.debug("GraphicsParameters state changed");
		}
		if (this.getDimension().getHeight() != oldGP.getDimension().getHeight()) {
			equal = false;
//			logger.debug("GraphicsParameters height changed");
		}
		if (this.getDimension().getWidth() != oldGP.getDimension().getWidth()) {
			equal = false;
//			logger.debug("GraphicsParameters width changed");
		}
		if (this.getLogaritmicMaximumValue() != oldGP.getLogaritmicMaximumValue()){
			equal = false;
//			logger.debug("GraphicsParameters logMax changed");
		}
		if (this.getLogaritmicMinimumValue() != oldGP.getLogaritmicMinimumValue()){
			equal = false;
//			logger.debug("GraphicsParameters logMin changed");
		}		
		return equal;
	}
	
//	public int hashCode() {
//		int hash =
//		
//	}
	/**
	 *  Use default settings pr. indicator for indicators with separate panel
	 * @return
	 */
	public int getBottomMargin() {
		return bottomMargin;
	}

	/**
	 * @return
	 */
	public Dimension getDimension() {
		return dim;
	}

	/**
	 * @return
	 */
	public int getLeftMargin() {
		uP =UserPreferences.getInstance();
		return uP.getLMargin();
	}

	/**
	 * @return
	 */
	public int getPixelPrDay() {
		uP =UserPreferences.getInstance();
		return uP.getPixelPrStick();
	}

	/**
	 * @return
	 */
	public int getRightMargin() {
		uP =UserPreferences.getInstance();
		return uP.getRMargin();
	}

	/**
	 * Use default settings pr. indicator for indicators with separate panel
	 * @return
	 */
	public int getTopMargin() {
		return topMargin;
	}

	/**
	 * @param dimension
	 */
	public void setDimension(Dimension dimension) {
		dim = dimension;
	}

	public int getNetWidth() {
		return (int) this.getDimension().getWidth() - this.getLeftMargin() - this.getRightMargin();
	}



	/**
	 * @return
	 */
	public double getLogaritmicMaximumValue() {
		return logaritmicMaximumValue;
	}

	/**
	 * @return
	 */
	public double getLogaritmicMinimumValue() {
		return logaritmicMinimumValue;
	}

	/**
	 * @param d
	 */
	public void setLogaritmicMaximumValue(double d) {
		logaritmicMaximumValue = d;
	}

	/**
	 * @param d
	 */
	public void setLogaritmicMinimumValue(double d) {
		logaritmicMinimumValue = d;
	}
	
	private void setState(){
		topMargin = this.getTopMargin();
		bottomMargin = this.getBottomMargin();
		//leftMargin = this.getLeftMargin();
		//rightMargin = this.getRightMargin();
		//pixelPrDay = this.getPixelPrDay();
	}
	
	private boolean stateChanged(GraphicsParameters oldGP) {
		boolean changed = false;
		if (this.getTopMargin() != oldGP.getTopMargin()) {
			changed = true;
		}
		if (this.getBottomMargin() != oldGP.getBottomMargin()){
			changed = true;
		}
		if (this.getLeftMargin() != oldGP.getLeftMargin()){
			changed = true;
		}
		if (this.getRightMargin() != oldGP.getRightMargin()){
			changed = true;
		}
		if (this.getPixelPrDay() != oldGP.getPixelPrDay()){
			changed = true;
		}
		return changed;
	}

	/**
	 * @param bottomMargin The bottomMargin to set.
	 */
	public void setBottomMargin(int bottomMargin) {
		this.bottomMargin = bottomMargin;
	}
	/**
	 * @param topMargin The topMargin to set.
	 */
	public void setTopMargin(int topMargin) {
		this.topMargin = topMargin;
	}
//	/**
//	 * @return Returns the displayWindow.
//	 */
//	public DisplayWindow getDisplayWindow() {
//		return m_displayWindow;
//	}
//	/**
//	 * @param displayWindow The displayWindow to set.
//	 */
//	public void setDisplayWindow(DisplayWindow displayWindow) {
//		this.m_displayWindow = displayWindow;
//	}

	/**
	 * Contains information about the position in a dataseries of the part of 
	 *	the indicator that is displayed on the screen. 
	 * 
	 * The numbers are relative to the end of the indicators series as this is the 
	 * only known point where the date is equal for all indicator series (e.g. the 
	 * indicatorseries for RSI(14) will be 14 days shorter than the 
	 * logDataSeries).
	 * 
	 * The relative start and end points will allways be equal or less than zero.
	 * The real start and end points will be:
	 * indicatorSeries.size -1 - relativeStart(End)Point. 
	 * 
	 * Using this allows for easy implementation of displaying other parts
	 * of the series than what is currently used.	
	 *  
	 * @param relativeEnd
	 */
	public void setRelativeEndOfPeriode(int relativeEnd) {
		m_relativeEndOfPeriode = relativeEnd;
	}
	/**
	 * See setRelativeEndOfPeriode(int relativeEnd)
	 * @return
	 */
	public int getRelativeEndOfPeriode() {
		return m_relativeEndOfPeriode;
	}
	/**
	 * See setRelativeEndOfPeriode(int relativeEnd)
	 * @return
	 */
	public int getRelativeStartOfPeriode() {
		int temp = m_relativeEndOfPeriode - (dim.width - this.getLeftMargin() - this.getRightMargin()) / this.getPixelPrDay();
		return temp;
	}
	
}
