/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 01-Nov-2004
 * 
 */
package com.exponto.jstock.gui.panels;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Transparency;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.indicators.CandleStick;
import com.exponto.jstock.indicators.Indicator;


/**
 * The MainIndicatorPanel is currently used for drawing the candlesticks.
 * 
 * It is larger than the other panels.
 * 
 * The CandleStick indicator needs a little special treatment, as it is possible 
 * for other indicators to be drawn into the same panel. But it also have quite
 * a few thing in common with the rest of the indicators. That's why it gets it 
 * own subclass. 
 * 
 * @author kaare
 *
 */
public class MainIndicatorPanel extends AbstractIndicatorPanel {
	
	//private ArrayList imageList = new ArrayList();
//	private int tMargin;
	private int lMargin;

	/**
	 * @param chartData
	 * @param ind
	 */
	public MainIndicatorPanel(ChartData chartData, Indicator ind) {
		super(chartData, ind);
		// Initiate certain candlestick spesific parameters
		CandleStick.setVolumeScale(userPrefs.getScalingFactorChartVolume());
		CandleStick.setMinorLineLevel(userPrefs.getMinorLineLevel());
		//m_tMargin = m_graphicsParameters.getTopMargin();
		//lMargin = m_graphicsParameters.getLeftMargin();
	}

    public void setGraphicsParameters (GraphicsParameters gP) {
		m_graphicsParameters = gP;
		this.setPreferredSize(gP.getDimension());
		this.setMaximumSize(gP.getDimension());
		lMargin = m_graphicsParameters.getLeftMargin();
		m_tMargin = m_graphicsParameters.getTopMargin();
		m_bMargin = m_graphicsParameters.getBottomMargin();
		//logger.debug("GraphicsParameters (dim, top, bottom margin"+ gP.getDimension().toString() + ", " + graphicsParameters.getTopMargin() + ", " + graphicsParameters.getBottomMargin());
    }

 
    
	public void paint(Graphics gr) {
		Graphics2D gr2d = (Graphics2D) gr;
		super.paint(gr2d);
		//logger.debug("IndicatorPanel, real width, height: " + this.getWidth() + ", " + this.getHeight());
		// Check if additional indicators (like MA & EMA) has changed
		ArrayList indicatorList = m_cD.getEnabledCandleStickIndicators();
		boolean indicatorChanged = false;
		for (int i = 0; i < indicatorList.size(); i++) {
			Indicator indicator = (Indicator) indicatorList.get(i);
			if (indicator.hasChanged()) {
				indicatorChanged = true;
			}
		}
		if (!this.imageValid(m_indicator) || indicatorChanged) {
			//System.out.println("Indicator size is " + m_graphicsParameters.getDimension());
			int width =  (int) m_graphicsParameters.getDimension().getWidth();
			int height = (int) m_graphicsParameters.getDimension().getHeight();
			m_bImage = gr2d.getDeviceConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
			// Get the Graphics
			Graphics2D bImageGraphics = m_bImage.createGraphics();
			bImageGraphics.setBackground(Color.lightGray);
			// Paints the candlesticks
			m_indicator.paintIndicator(bImageGraphics, m_graphicsParameters);
			// Paints additional indicators
			String indicatorString = null;
			int y = m_graphicsParameters.getDimension().height + m_tMargin - 35; // Below panel
			int x = m_graphicsParameters.getNetWidth() + lMargin; // To the right
			int length;
			Line2D line = new Line2D.Double();
			for (int i = 0; i < indicatorList.size(); i++) {
				Indicator indicator = (Indicator) indicatorList.get(i);
				indicator.initiateIndicator(m_cD.getLogDataSeries());
				indicator.paintIndicator(bImageGraphics, m_graphicsParameters);
				// Draws Indicatorname and line color to the right of the display 
				bImageGraphics.setColor(indicator.getColor());			
				line.setLine(x - 25 , y -5, x , y - 5);
				bImageGraphics.draw(line);
				indicatorString = indicator.toString();
				length = indicatorString.length() * 8;
				x = x - length - 25;
				bImageGraphics.setColor(Color.black);
				bImageGraphics.drawString(indicatorString, x , y);
				x = x -10;
			}
		}
		gr2d.drawImage(m_bImage, 0, 0, this);
		// Draws lines
		this.drawLines(gr2d);
		// Paint the status line at the top of the candlestick panel 
		 drawStatusField(gr2d, lMargin, m_tMargin);
		 // Print additional indicator information
		drawIndicatorStatusField(gr2d, lMargin, m_tMargin);
	}
	
	
	private void drawIndicatorStatusField(final Graphics2D g2d, int lMargin, int tMargin) {
		String indicators = "";
		int numberOfSticksToDisplay = m_graphicsParameters.getNetWidth() / userPrefs.getPixelPrStick();
		if (m_dailyPrintout >= 0 &&  m_dailyPrintout < numberOfSticksToDisplay) {
			//Print selected
			indicators = this.getIndicatorText(m_dailyPrintout);
		} else {
			// Print last value
			indicators = this.getIndicatorText(numberOfSticksToDisplay);
		}
		int i = m_graphicsParameters.getDimension().height + tMargin - 35;
		//System.out.println("MainIndicatorPanel, line 162 - numberOfSticksToDisplay, m_dailyPrintout: " + numberOfSticksToDisplay + ", " + m_dailyPrintout);
		g2d.drawString(indicators, lMargin, m_graphicsParameters.getDimension().height + tMargin - 35);
	}
	
	private String getIndicatorText(int dayNumber) {
		ArrayList indicatorList = m_cD.getEnabledCandleStickIndicators();
		String indicatorPrintOut = "";
		for (int i = 0; i <indicatorList.size();i++) {
			Indicator indicator = (Indicator) indicatorList.get(i);
			indicatorPrintOut = indicatorPrintOut + indicator.getDailyPrintOut(dayNumber) + "   ";
		}
		return indicatorPrintOut;
	}
	
	/**
	 * Draw Status field at top of the display
	 * 
	 * @param g2d
	 */
	private void drawStatusField(final Graphics2D g2d, int lMargin, int tMargin) {
		g2d.drawString(m_indicator.getDailyPrintOut(m_dailyPrintout) + m_indicator.toString(), lMargin, tMargin-6);
		// TODO: Works, but not very clean to use CandleStick class 
		CandleStick  cS = (CandleStick) m_indicator;
		cS.markSelectedStick(g2d, m_dailyPrintout, lMargin, tMargin);
	}

	
//	private void drawIndicators(final Graphics2D g2d, int lMargin, int tMargin) {
//	ArrayList indicatorList = m_cD.getEnabledCandleStickIndicators();
//	if (imageList.size() != indicatorList.size()) {
//		// Initialising the imageList with null in order to avoid any problems with get/set and 0pointers  
//		imageList.clear();
//		for (int i = 0; i <indicatorList.size();i++) {
//			Indicator indicator = (Indicator) indicatorList.get(i);
//			indicator.initiateIndicator(m_cD.getLogDataSeries());
//			imageList.add(null);
//		}
//		logger.debug("ImageList is reset");
//	}
//	int y = m_graphicsParameters.getDimension().height + tMargin + 35;
//	int x = m_graphicsParameters.getDimension().width + lMargin;
//	int length;
//	Line2D line = new Line2D.Double();
//	String indicatorString;
//	//BufferedImage bufferedImage;
//	for (int i = 0; i <indicatorList.size();i++) {
//		m_bImage = (BufferedImage) imageList.get(i);
//		Indicator indicator = (Indicator) indicatorList.get(i);
//		if (!this.imageValid(indicator)) {
//			int iWidth =  (int) m_graphicsParameters.getDimension().getWidth();
//			int iHeight = (int) m_graphicsParameters.getDimension().getHeight();
//			m_bImage = g2d.getDeviceConfiguration().createCompatibleImage(iWidth, iHeight, Transparency.TRANSLUCENT);
//			// Get the Graphics
//			Graphics2D bImageGraphics = m_bImage.createGraphics();
//			//bImageGraphics.setBackground(Color.lightGray);
//			indicator.initiateIndicator(m_cD.getLogDataSeries());
//			indicator.paintIndicator(bImageGraphics, m_graphicsParameters);
////			bImage = this.drawImage(indicator);
//			imageList.set(i, m_bImage);
//			//logger.debug("Indicator not valid");
//		}
//		g2d.drawImage(m_bImage, 0, 0, this);
//		g2d.setColor(indicator.getColor());			
//		line.setLine(x - 25 , y -5, x , y - 5);
//		g2d.draw(line);
//		indicatorString = indicator.toString();
//		length = indicatorString.length() * 8;
//		x = x - length - 25;
//		g2d.setColor(Color.black);
//		g2d.drawString(indicatorString, x , y);
//		x = x -10;
//	}
//}
	
}
