/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 29-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.text.DecimalFormat;
import java.util.ArrayList;

//import com.exponto.util.FifoQueue;
import com.exponto.jstock.gui.panels.GraphicsParameters;
//import com.exponto.jstock.LogTradingData;

/**
 * @author kaare
 *
 */
public class EMA extends Indicator {

	static SingleParameterParameters m_defaultParameters = null;
	SingleParameterParameters m_parameters = null;
	private static final String propName = "EMA.NumberOfDays";
	private static final String toolTip = "Enter the number of days to use in Exponential Moving Average calculation. Use 0 for no EMA.";
	GraphicsParameters graphicsParameters;
	protected ArrayList indicatorSeries;
	
	//int lMargin;
	//int rMargin;
	//int widthOfDay;
	int numberOfDaysToDisplay;
	double scaleY = 0.01;
	double logMin;
	double logMax;
	//private static Dimension currentRSIDim;
	int width;
	int height;
	int x0;
	
	//int oldNumberOfDays = 0;
	//boolean parametersChanged = true;
	private int m_oldNumberOfDays;
	private int m_startOffset;

	/**
	 * 
	 */
	public EMA() {
		super();
		indicatorSeries = new ArrayList();
		sm_indicatorName = Constants.ema;
		if (m_defaultParameters == null) {
			m_defaultParameters = new SingleParameterParameters(14, propName, sm_indicatorName, toolTip);
		}
		sm_indicatorColor = Color.green;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getIndicatorParameters()
	 */
	public IndicatorParameters getIndicatorParameters() {
		if (m_parameters == null) {
			return m_defaultParameters;
		}
		return m_parameters;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		if (m_parameters == null) {
			if (m_oldNumberOfDays != m_defaultParameters.getNumberOfDays()) {
				// Only update if parameters has changed
				indicatorSeries = EMAAlgorithm.computeEMA(lDS, m_defaultParameters.getNumberOfDays());
				m_startOffset = findStartOffset(indicatorSeries);
			}
			//m_parametersChanged = true;
			//logger.debug("Default indicatorSeries for EMA used");
		} else {
			if (m_oldNumberOfDays != m_parameters.getNumberOfDays()) {
				// Only update if parameters has changed
				indicatorSeries = EMAAlgorithm.computeEMA(lDS, m_parameters.getNumberOfDays());
				m_startOffset = findStartOffset(indicatorSeries);
			}
			//m_parametersChanged = true;
			//logger.debug("Specific indicatorSeries for EMA used");
		}
	}

//	/* (non-Javadoc)
//	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
//	 */
//	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
//		int [] periode = indParam.getParameters(); 
//		indicatorSeries = EMAAlgorithm.computeEMA(lDS, periode[0]);
//		parametersChanged = true;
//		//oldNumberOfDays = emaParameters.getNumberOfDays();
//	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.Indicator#initiateIndicator(com.exponto.jstock.LogDataSeries, java.lang.String)
	 */
	public void initiateIndicator(LogDataSeries lDS, String indParam) {
		SingleParameterParameters tempEMAParameters = new SingleParameterParameters(indParam, propName, sm_indicatorName, toolTip);
		if (m_oldNumberOfDays != tempEMAParameters.getNumberOfDays() || m_parameters == null){
			// Uses m_rsiParameters if non existent and updates if parameters has changed
			m_parameters = tempEMAParameters;
			indicatorSeries = EMAAlgorithm.computeEMA(lDS, m_parameters.getNumberOfDays());
			m_startOffset = findStartOffset(indicatorSeries);
			//m_indicatorSeries = loadRsi(lDS, m_Parameters);
			//m_parametersChanged = true;
			logger.debug("indicatorSeries for EMA has changed");
		}
		//m_parameters = new SingleParameterParameters(indParam, propName, sm_indicatorName, toolTip);
		//indicatorSeries = EMAAlgorithm.computeEMA(lDS, m_parameters.getNumberOfDays());
		//parametersChanged = true;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return true;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D, com.exponto.jstock.GraphicsParameters)
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP) {
		graphicsParameters = gP;
		logMin = graphicsParameters.getLogaritmicMinimumValue();
		logMax = graphicsParameters.getLogaritmicMaximumValue();
		int tMargin = graphicsParameters.getTopMargin();
		int bMargin = graphicsParameters.getBottomMargin();
		
		width =  gP.getNetWidth();
		height = (int) gP.getDimension().getHeight() - tMargin - bMargin;
		x0 = gP.getLeftMargin();
		int yma0 = height + tMargin;
		scaleY =  height/(logMax - logMin);
//		logger.debug("ema area - height, scaleY, yma0: " + height + ", " + scaleY + ", " + yma0);
//		logger.debug("Indicator  LogMin, LogMax: "+ logMin + ", " + logMax);
		int widthOfDay =gP.getPixelPrDay();
		numberOfDaysToDisplay = width / widthOfDay;
		g2d.setColor(Color.yellow);
		
		int emaDays = m_defaultParameters.getNumberOfDays();
		// Skip if size is zero 
		if (indicatorSeries.size() == 0) return;
		Double value = null;
		double yma, yma2, ytemp;
		int emaLength = indicatorSeries.size() - emaDays; // This is the number of valid values in emaList
		int xma =  x0 + widthOfDay/2;
		if (emaLength < numberOfDaysToDisplay) {
			// Adjust starting point if dataseries is shorter than display
			if ((emaLength + emaDays) < numberOfDaysToDisplay) {
				// Also the dataseries is shorter than the display 
				xma = xma +  widthOfDay * (m_startOffset);
				// Start at first valid value
				emaLength = m_startOffset;
				//System.out.println("Startoffset EMA: " + (m_startOffset));
			} else {
				// Only the maList is shorter than the display
				xma = xma + widthOfDay * (numberOfDaysToDisplay - emaLength + 1);
				emaLength = emaDays + m_startOffset;
			}
		} else {
			// Starting point within emaList when valid emaList is longer than display
			emaLength = indicatorSeries.size() - numberOfDaysToDisplay - 1;	
		}

		Line2D maLine = new Line2D.Double();
		g2d.setColor(sm_indicatorColor);
//		logger.debug("RsiPanel - ArraySize, Days to display: " + rsiList.size() + ", " + numberOfDaysToDisplay);
		
//		Ensure that i is within array bounds
		if (emaLength < 0 ) {
			logger.debug("Set emaLength to 0");
			emaLength = 0;
		} else if (emaLength >= indicatorSeries.size()) {
			emaLength = indicatorSeries.size() -1;
		}

		while (value == null) {
			value = (Double) indicatorSeries.get(emaLength ++);
		}
		ytemp = Math.log(value.doubleValue())/Constants.log10 - logMin;
		yma = yma0  - ytemp * scaleY ;

		for (int i = (emaLength); i < indicatorSeries.size(); i++) {
			value = (Double) indicatorSeries.get(i);
			if (value != null) {
				ytemp = Math.log(value.doubleValue())/Constants.log10 - logMin;
				yma2 = yma0  - ytemp * scaleY ;
				maLine.setLine(xma, yma, xma + widthOfDay, yma2);
				g2d.draw(maLine);
				//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
				yma = yma2;
			}
			xma = xma + widthOfDay;
		}
		g2d.setColor(Color.black);
		// Update status variables
		//parametersChanged = false;
		if (m_parameters == null) {
			m_oldNumberOfDays = m_defaultParameters.getNumberOfDays();	
		} else {
			m_oldNumberOfDays = m_parameters.getNumberOfDays();
		}



	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		String s = " = ";
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(2);
		df.setDecimalSeparatorAlwaysShown(false);
		String value ="NA";
		
		int index = 0;
		if (n < 0) {
			index = indicatorSeries.size() -1;
			s = ", last value is ";
		} else {
			//System.out.println("index: " + index);
			int displayWidth = graphicsParameters.getNetWidth()/ graphicsParameters.getPixelPrDay();
			if (indicatorSeries.size() >= displayWidth) {
				// Series is longer than display width
				index = indicatorSeries.size() -1 - (displayWidth);
			}
			//System.out.println("index I: " + index + " Series, display  length: " + indicatorSeries.size() + ", " + displayWidth);
			index = n + index;
		}
		
		//int index = displayNumberToArrayIndex(n, graphicsParameters, indicatorSeries);
		if (index < indicatorSeries.size() && index >= 0) {
			Double  d = (Double) indicatorSeries.get(index);
			if (d != null) {
				value = df.format(d);
			}
		}
		return this.toString() + s + value;
	}

	public boolean hasChanged() {
	 	boolean result = true;
	 	if (m_parameters == null) {
	 		if (m_oldNumberOfDays == m_defaultParameters.getNumberOfDays()) {
	 			result = false;
	 		}
	 	} else {
	 		if (m_oldNumberOfDays == m_parameters.getNumberOfDays()) {
	 			result = false;
	 		}
	 	}
	 	return result;
	}

	public String toString() {
		if (m_parameters == null) {
			return sm_indicatorName +"(" + m_defaultParameters.getNumberOfDays() + ")";
		}
		return sm_indicatorName +"(" + m_parameters.getNumberOfDays() + ")";
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int)
	 */
	public String[] printAll(int number) {
		Double d;
		String [] sA = new String[number + 1];
		int start = indicatorSeries.size() - number;
		int index = 1;
		sA[0] = "EMA("+String.valueOf(m_defaultParameters.getParameters())+")";
		if (start < 0) start = 0;
		for (int i = start; i < indicatorSeries.size(); i++) {
			d = (Double) indicatorSeries.get(i);
			//d = (Double) indicatorSeries.get(displayNumberToArrayIndex(i, graphicsParameters, indicatorSeries));
			sA[index++] = d.toString();
		}
		return sA;
	}

	public boolean supportIndicatorValuesInterface() {
		return false;
	}
	/**
	 * Counts number of null elements in the begining of a indicator series
	 * @param indSeries
	 * @return
	 */
	private int findStartOffset(ArrayList indSeries) {
		int offset = 0;
		int i = 0;
		while(indSeries.get(i++) == null) {
			offset ++;
		}
		return offset;
	}
}
