/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 06-Mar-2004
 *
 
 */
package com.exponto.jstock.indicators;

import com.exponto.jstock.gui.panels.GraphicsParameters;

import java.awt.Graphics2D;
import java.awt.Color;
import java.util.ArrayList;
import org.apache.log4j.Logger;

/**
 * The base class for all indicators. A few methods of the indicator
 * interface are implemented here, the rest are found in the subclasses that
 * implements the actual indicators.
 * 
 * A indicator must at least implement the interfaces IndicatorInterface and
 * IndicatorParametersInterface. The first for providing basic indicator 
 * "services" and the last in order to make the user able to modify the 
 * indicator parameters.
 * 
 * @author kaare
 *
 */
public abstract class Indicator implements IndicatorInterface {
		
	protected String sm_indicatorName;
	protected Color sm_indicatorColor;
	protected boolean sm_isDisplayed = true; // Default is that all created indicators shall be visible
	
	static Logger logger = Logger.getLogger(Indicator.class);

	public Indicator() {
		//indicatorSeries = new ArrayList();
	}
	
	public abstract IndicatorParameters getIndicatorParameters();
	
	public abstract void initiateIndicator(LogDataSeries lDS);

	//public abstract void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam);
	
	public abstract void initiateIndicator(LogDataSeries lDS, String indParam);

	public boolean getDisplayed() {
		return sm_isDisplayed;
	}
	
	public void setDisplayed(boolean b) {
		sm_isDisplayed = b;
	}
	
	public boolean isMainIndicator() {
		return false;
	}
	
	public abstract boolean drawWithCandleSticks();
	
	public abstract boolean hasChanged();

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paint(java.awt.Graphics2D)
	 */
	public abstract void paintIndicator(Graphics2D g2d, GraphicsParameters gP);
	
	public abstract String getDailyPrintOut(int n);
	
	public abstract String[] printAll(int number);

	public Color getColor() {
		return sm_indicatorColor;
	}
	/**
	 * 
	 * @param color - Sets the indicators color. Note that some indicators 
	 * uses more than one color, this is the "main" color.
	 */
	public void setColor(Color color) {
		sm_indicatorColor = color;
	}
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getIndicatorName()
	 */
	public String getIndicatorName() {
		return sm_indicatorName;
	}

	/**
	 * Avoid using this as it is not correct in all cases - will be removed
	 * Finds the place in the indicatorSeries for a particular posistion on the display
	 * 
	 * @param n the posistion on display
	 * @return the position in the indicatorSeries
	 */	
	protected int displayNumberToArrayIndex(int n, GraphicsParameters graphicsParameters, ArrayList indicatorSeries) {
		int result;
		//logger.debug("Netwidth "  + ", " + graphicsParameters.getPixelPrDay());
		int numberOfDays = graphicsParameters.getNetWidth() / graphicsParameters.getPixelPrDay();
		int length = indicatorSeries.size();
		if (length >= numberOfDays) {
			result = length -1 - numberOfDays + n;
		} else {
			// Not correct for all cases see e.g. dnbnor, mostly not used any more
			result = n - (numberOfDays - length - 1);
		}
		return result;
	}
}
