/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;
import javax.swing.JComponent;
import com.exponto.util.Date;
import org.apache.log4j.Logger;
/**
 * Base class for indicator specific parameters used in computation of an
 * indicator series.
 *
 *  
 * @author kaare
 *
 */
public abstract class IndicatorParameters implements IndicatorParametersInterface {
	protected Date m_validFrom;
	protected Date sm_validTo;
	protected String sm_delimiter ="/";
	static Logger logger = Logger.getLogger(IndicatorParameters.class);
	protected int [] m_parameterValues;
	protected JComponent [][] m_compArray;

	public int [] getParameters() {
		return m_parameterValues;
	}

	/**
	 * 
	 * 
	 * @param parameters
	 */
	public void setParameters(int [] parameters) {
		m_parameterValues = parameters;
	}	
	/**
	 * 
	 * @param index
	 * @param value
	 */
	public void setParameters(int index, int value) {
		m_parameterValues[index] = value;
	}

	public abstract String [] getParameterNames();
	
	public abstract boolean indicatorEnabled();
	
	/**
	 * Provides the Options menu with components to use for this indicator.
	 * The JComponents are responsible for notifying "their" indicator when 
	 * values are changed, and also update the properties files.
	 * 
	 * Note that the receiving class makes a few assumptions on the 
	 * content of the array.
	 * 
	 * @return an JComponent array with the components that shall be 
	 * displayed
	 */
	public abstract JComponent[][] getUserInterface();

	public abstract void updateParameters();
	
	public void setValidFrom(Date date) {
		m_validFrom = date;
	}

	/**
	 * @return the first date with valid data for the indicator 
	 */
	public Date getValidFrom() {
		return m_validFrom;
	}


	public void setValidTo(Date date) {
		sm_validTo = date;
	}

	/**
	 * @return the last date with valid data for the indicator 
	 */
	public Date getValidTo() {
		return sm_validTo;
	}

	public boolean equals (Object o) {
		IndicatorParameters other = (IndicatorParameters) o;
		if (other == null) return false;
		boolean result = true;
		int [] otherParameters = other.getParameters();
		int [] parameters = this.getParameters();
		if (parameters.length != otherParameters.length) {
			// Length not equals
			return false;
		}
		for (int index = 0; index < parameters.length; index ++) {
			// Check if equal
			if (parameters[index] != otherParameters[index]) {
				result = false;
			}
		}
		return result;
	}
	/**
	 *Note:  Reach MAXINT after about 7 - 10 parameters depending on the parameter
	 */
	public int hashCode() {
		int result = 0;
		int [] parameters = this.getParameters();
		for (int index = 0; index < parameters.length; index ++) {
			result = result + parameters[index] * ((int) Math.pow(10, index) );
		}
		return result;
	}
	
	
	public String toString() {
		String s = null;
		int [] parameters = this.getParameters();
		if (parameters.length > 0) {
			s = String.valueOf(parameters[0]);
			for (int index = 1; index < parameters.length; index ++) {
				s = s + sm_delimiter + parameters[index];
			}
		}
		return s; 
	}

}
