/*
 * Created on 12-Oct-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JFrame;

import org.apache.log4j.PropertyConfigurator;

import com.exponto.jstock.jStock;
import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.gui.panels.GraphicsParameters;
import com.exponto.jstock.io.TickerID;

import junit.framework.TestCase;

/**
 * @author kaare
 *
 */
public class IndicatorTest extends TestCase {
	String ticker = "nsg";
	String exchange = "ose";
	LogDataSeries lDS;
	ChartData chartData;
	TickerID tId;
	GraphicsParameters gP;
	
	public IndicatorTest(String arg0) {
		super(arg0);
		//this.runTest();
	}

	public void runTest() {
		testIndicator();
	}
	
	public void testIndicator() {
		hereItIs("rsi", "11");
		hereItIs("ema", "13");
		hereItIs("ma", "17");
		hereItIs("macd", "17/34/11");
	}
	/**
	 * 
	 * @param indName name of the indicator to test
	 * @param indParam a string representation of the indicators parameters
	 */
	public void hereItIs(String indName, String indParam) {
		System.out.println(indName);
		Indicator indicator = IndicatorMaster.getIndicator(indName);
		assertTrue(indicator != null);
		indicator.initiateIndicator(lDS, indParam);
		IndicatorParameters indicatorParameters = indicator.getIndicatorParameters();
		// Verify that the parameter is as expected 
		assertTrue(indParam.equals( indicatorParameters.toParameterString()));
		// Verify that the indicator isn't previously computed with theese parameters
		assertTrue(indicator.hasChanged());
		// Need a Graphics2D
		JFrame jP = new JFrame();
		JButton quit = new JButton("Quit");
		jP.getContentPane().add(quit);
		jP.setVisible(true);
		Graphics2D g2d =  (Graphics2D) jP.getGraphics();
		indicator.paintIndicator(g2d, gP);
		// Verify that indicator has not changed since last paint
		assertFalse(indicator.hasChanged());
		
		//System.out.println(indicatorParameters.toParameterString());
		
	}
	
	protected void setUp() throws Exception {
		super.setUp();
		// Setup logger
		String resource ="/log.properties";
		URL configFileResource = jStock.class.getResource(resource);
		PropertyConfigurator.configure(configFileResource);    	
		// Get some data to work with 
		tId = new TickerID(ticker,exchange);
		chartData = new ChartData(tId, "");
		lDS = chartData.getLogDataSeries();
		gP = new GraphicsParameters();
		// Some valid default values
		gP.setDimension(new Dimension(200,300));
		gP.setBottomMargin(10);
		gP.setTopMargin(10);
		gP.setLogaritmicMaximumValue(1000);
		gP.setLogaritmicMinimumValue(0.01);
	}
	public static void main(String[] args) {
		junit.textui.TestRunner.run(IndicatorTest.class);
	}

}
