/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 08-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.Color;
import java.util.ArrayList;
//import java.text.DecimalFormat;

import com.exponto.jstock.gui.panels.GraphicsParameters;
/**
 * @author kaare
 *
 */
public class Macd extends Indicator implements IndicatorLinesInterface {
	final static double log10 = Math.log(10);
	private static MacdParameters m_defaultParameters;
	private MacdParameters m_parameters;
	MacdParameters oldMacdParameters;
	GraphicsParameters graphicsParameters;
	IndicatorAxisInfo m_indicatorAxisInfo = null;
	protected ArrayList indicatorSeries;
//	protected ArrayList triggerSeries;
//	protected ArrayList histogramSeries;
//	boolean parametersChanged = true;

//	int tMargin = 0;
//	int bMargin = 0;
	int widthOfDay;
	int numberOfDaysToDisplay;
	double scaleY = 0.01;
	int width;
	int height;
	int x0, y0;
	Color indicatorColorHistogram;
	Color indicatorColorTrigger;
	//private IndicatorAxisInfo m_iAI = null;
	
	public Macd () {
		super();
		sm_indicatorName = Constants.macd;
		indicatorSeries = new ArrayList();
		if (m_defaultParameters == null) {
			m_defaultParameters = new MacdParameters(12, 26, 9);	
		}
		m_indicatorAxisInfo = new IndicatorAxisInfo(0.0, 0.0, Constants.linear); //  dummy min-max values
		sm_indicatorColor = Color.black;
		indicatorColorHistogram = Color.yellow;
		indicatorColorTrigger = Color.gray; 
	}

	public IndicatorParameters getIndicatorParameters() {
		if (m_parameters == null) {
			return m_defaultParameters;
		}
		return m_parameters;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(java.lang.String, com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		if (m_parameters == null) {
			if (!m_defaultParameters.equals(oldMacdParameters)) {
				// Only update if parameters has changed
				indicatorSeries = loadMacd(lDS, m_defaultParameters);	
			}
			//logger.debug("Default indicatorSeries for MACD used");
		} else {
			if (!m_parameters.equals(oldMacdParameters)) {
				// Only update if parameters has changed
				indicatorSeries = loadMacd(lDS, m_defaultParameters);	
			}
			//logger.debug("Specific indicatorSeries for MACD used");
		}
//		if (!m_defaultParameters.equals(oldMacdParameters)){
//			indicatorSeries = loadMacd(lDS, m_defaultParameters);
//		}
	}
	
//	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
//		if (!indParam.equals(oldMacdParameters)){
//			indicatorSeries = loadMacd(lDS, (MacdParameters) indParam);
//		}
//	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.Indicator#initiateIndicator(com.exponto.jstock.LogDataSeries, java.lang.String)
	 */
	public void initiateIndicator(LogDataSeries lDS, String indParam) {
		MacdParameters tempParameters = new MacdParameters(indParam);
		if (!tempParameters.equals(m_parameters)){
			// Uses m_parameters if non existent and updates if parameters has changed
			m_parameters = tempParameters;
			indicatorSeries = loadMacd(lDS, m_parameters);
			//m_parametersChanged = true;
			logger.debug("indicatorSeries for MACD has changed");
		}
//		m_parameters = new MacdParameters(indParam);
//		indicatorSeries = loadMacd(lDS, m_parameters);
//		parametersChanged = true;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return false;
	}
	
	 public boolean hasChanged() {
	 	boolean result = true;
	 	if (m_parameters == null) {
	 		if (m_defaultParameters.equals(oldMacdParameters)) {
	 			result = false;
	 		}
	 	} else {
	 		if (m_parameters.equals(oldMacdParameters)) {
	 			result = false;
	 		}
	 	}
	 	return result;
	 	//return parametersChanged;
	 }
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D)
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP) {
		graphicsParameters = gP;
		width =  gP.getNetWidth();
		height = (int) gP.getDimension().getHeight() - gP.getTopMargin() - gP.getBottomMargin();
		x0 = gP.getLeftMargin();
		widthOfDay =gP.getPixelPrDay();
		numberOfDaysToDisplay = width / widthOfDay;
		if (indicatorSeries == null) {
			return;
		}
		int start = indicatorSeries.size() - numberOfDaysToDisplay -1;
		if (start < 0) {
			start = 0;
		}
		// Need to find hig & low in the series in order to get the scaling right
		double low = 9999999999999999999999999999.9;
		double high = - 999999999999999999999999999.9;
		MacdData macdData; 
		for (int i = start; i < indicatorSeries.size(); i++) {
			macdData = (MacdData) indicatorSeries.get(i);
			if (macdData != null) {
				if (macdData.getMacd() > high) {
					high = macdData.getMacd();
				}
				if (macdData.getMacd() < low) {
					low = macdData.getMacd();
				}
				if (macdData.getTrigger() > high) {
					high = macdData.getTrigger();
				}  
				if (macdData.getTrigger() < low) {
					low = macdData.getTrigger();
				}
				if (macdData.getHistogram() > high) {
					high = macdData.getHistogram();
				} 
				if (macdData.getHistogram() < low) {
					low = macdData.getHistogram();
				}
			}
		}
//		// Find high and low values for the graph
//		// Uses "logaritmic" steps (i.e. +/- 0.1, 1, 10 and so on)
//		double base = (Math.log( - low)/log10);
//		base = Math.floor(base) +1;
//		double newLow =  - Math.pow(10, base);
//		base = (Math.log(high)/log10);
//		base = Math.floor(base) + 1;
//		double newHigh = Math.pow(10, base);
//		// To ensure that scale doesn't get way of
//		// A quick hack
//		if (newHigh/high > 2) {
//			newHigh = newHigh/2;
//		}
//		if (newLow/low > 2) {
//			newLow = newLow/2;
//		}
		
//		double absoluteHeight = newHigh - newLow;
		m_indicatorAxisInfo.setMinValue(low);
		m_indicatorAxisInfo.setMaxValue(high);
		double absoluteHeight = high - low;
		//logger.debug("MACD high, low: " + newHigh + ", " + newLow);
		scaleY = height/absoluteHeight;
		double yZero = high/absoluteHeight;
		yZero = height * yZero + gP.getTopMargin();
		y0 = (int) Math.floor(yZero);
		//logger.debug("MACD y0: " + y0);
		//logger.debug("MACD Panel - height, scaleY, y0: " + height + ", " + scaleY + ", " + y0);
		//g2d.setBackground(Color.lightGray);
		g2d.setColor(Color.black);
		//drawTitle(g2d);
		drawAxis(g2d);
		if (indicatorSeries == null) {
			logger.error("indicatorSeries for MACD is null");
		} else if (indicatorSeries.size() == 0){
			logger.error("indicatorSeries for MACD has size of null");
		} else {
			drawMacd(g2d);			
		}
		// Update status variables
		//parametersChanged = false;
		if (m_parameters == null) {
			oldMacdParameters = (MacdParameters) m_defaultParameters.clone();
		} else {
			oldMacdParameters = (MacdParameters) m_parameters.clone();
		}

		//logger.debug("MACD panel repainted");
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		String s = " = ";
//		DecimalFormat df = new DecimalFormat();
//		df.setMaximumFractionDigits(2);
//		df.setDecimalSeparatorAlwaysShown(false);
		String value ="NA";
		int index = 0;
		if (indicatorSeries == null) {
			return this.toString() + s + value;
		}
		if (n < 0) {
			index = indicatorSeries.size() -1;
			s = ", last value is ";
		} else {
			int displayWidth = graphicsParameters.getNetWidth()/ graphicsParameters.getPixelPrDay();
			if (indicatorSeries.size() >= displayWidth) {
				// Series is longer than display width
				index = indicatorSeries.size() -1 - (displayWidth);
			}
			//System.out.println("index I: " + index + " Series, display  length: " + indicatorSeries.size() + ", " + displayWidth);
			index = n + index;
		}
		if (index < indicatorSeries.size() && index >= 0) {
			MacdData  macdData = (MacdData) indicatorSeries.get(index);
			if (macdData != null) {
				value = macdData.toString();
			}
		}
		return this.toString() + s + value;
	}

	public String toString() {
		if (m_parameters == null) {
			return sm_indicatorName +"(" + m_defaultParameters.toString() + ")";
		}
		return sm_indicatorName +"(" + m_parameters.toString() + ")";		
	}

//	public void printValue(int xValue){
//		String value = this.getDailyPrintOut(xValue);
//		
//	}
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int, int)
	 */
	public String[] printAll(int number) {
		Double d;
		String [] sA = new String[number + 1];
		int start = indicatorSeries.size() - number;
		int index = 1;
		sA[0] = "MACD("+m_defaultParameters.toString()+")";
		if (start < 0) start = 0;
		for (int i = start; i < indicatorSeries.size(); i++) {
			d = (Double) indicatorSeries.get(i);
			//d = (Double) indicatorSeries.get(displayNumberToArrayIndex(i, graphicsParameters, indicatorSeries));
			sA[index++] = d.toString();
		}
		return sA;
	}

	public boolean supportIndicatorValuesInterface() {
		return false;
	}

//	private void drawTitle(final Graphics2D g2d) {
//		// Title
//		String str = "MACD("+ macdParameters.toString()+")";
//		g2d.drawString(str, 5, (int) y0 - height);
//	}
	
	private void drawAxis(final Graphics2D g2d) {
		//x-axis
		Line2D line = new Line2D.Double (x0, y0, x0 + width, y0);
		g2d.draw(line);
		String str = String.valueOf(0);
		g2d.drawString(str, 15, y0+5);
		//y-axis
		line.setLine(x0, graphicsParameters.getTopMargin() + height, x0, graphicsParameters.getTopMargin());
		g2d.draw(line);
	}

	private void drawMacd(final Graphics2D g2d) {
		MacdData macdData;
		Line2D lineMacd = new Line2D.Double ();
		//Line2D lineTrigger = new Line2D.Double ();
		Rectangle2D rectHistogram = new Rectangle2D.Double(); 
		double yMacd, yMacd2;
		double yTrigger, yTrigger2;
		double yHist;
		int length;
		int x = x0 + widthOfDay/2;
		length = indicatorSeries.size() - numberOfDaysToDisplay - 1;
//		Ensure that i is within array bounds
		if (length < 0) {
			length = 0;
			logger.debug("Length is less than zero");
		}
		macdData = (MacdData) indicatorSeries.get(length++);
		while (macdData == null) { 
			macdData = (MacdData) indicatorSeries.get(length++);
			x = x + widthOfDay;
		}
		yMacd = y0  - macdData.getMacd() * scaleY;
		yTrigger = y0  - macdData.getTrigger() * scaleY;
		yHist = macdData.getHistogram() * scaleY;
		//logger.debug("hist, yHist, scaleY: " + macdData.getHistogram() + ", " + yHist + ", " + scaleY);
		for (int i = length; i < indicatorSeries.size(); i++) {
			macdData = (MacdData) indicatorSeries.get(i);
			if (macdData != null) {
				// Draw histogram
				g2d.setColor(indicatorColorHistogram);
				if (yHist > 0) {
					rectHistogram.setRect((x ), y0 - yHist, widthOfDay -1, yHist - 1); //- widthOfDay/2
				} else {
					rectHistogram.setRect((x ), y0 + 1, widthOfDay -1, - yHist);
				}
				//logger.debug("y0, yHist, scaleY: " + y0 + ", " + yHist + ", " + scaleY);
				g2d.draw(rectHistogram);
			}
//			macdData = (MacdData) indicatorSeries.get(i);
			if (macdData != null) {			
				yMacd2 = y0  - macdData.getMacd() * scaleY ;
				yTrigger2 = y0  - macdData.getTrigger() * scaleY;
				yHist = macdData.getHistogram() * scaleY;
				// Draw MACD
				g2d.setColor(sm_indicatorColor);
				//line.setLine(x, y, x + widthOfDay, y);
				lineMacd.setLine(x, yMacd, x + widthOfDay, yMacd2);
				g2d.draw(lineMacd);
				lineMacd.setLine(x, yMacd -1 , x + widthOfDay, yMacd2 - 1);
				g2d.draw(lineMacd);
				//Draw trigger
				g2d.setColor(indicatorColorTrigger);
				lineMacd.setLine(x, yTrigger, x + widthOfDay, yTrigger2);
				g2d.draw(lineMacd);
				//System.out.println("MACD Panel - MACD, trigger, hist: " + macdData.getMacd() + ", "+ macdData.getTrigger() + ", " + macdData.getHistogram());
				yMacd = yMacd2;
				yTrigger = yTrigger2;
			} //else {
				//logger.debug("macdData is null");
			//}
			x = x + widthOfDay; 
			
		}
		//logger.debug("MACD Panel - x, y, value: " + x + ", "+ y + ", " + value);
	}

	private ArrayList loadMacd(LogDataSeries lDS, MacdParameters macdParam) {
		//parametersChanged = true;
		oldMacdParameters = null;
		return MacdAlgorithm.computeMacd(lDS, macdParam);
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorLinesInterface#getIndicatorAxisInfo()
	 */
	public IndicatorAxisInfo getIndicatorAxisInfo() {
		return m_indicatorAxisInfo;
	}
	
}
