/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Contains MACD specific parameters used in computation of an
 * indicator series
 * 
 * @author kaare
 *
 */
public class MacdParameters extends IndicatorParameters implements Cloneable {
	JTextField macdSlowText;
	JTextField macdFastText;
	JTextField macdTriggerText;
	
	public MacdParameters(int fast, int slow, int trigger) {
		m_parameterValues = new int[3];
		m_parameterValues[0] = fast;
		m_parameterValues[1] = slow;
		m_parameterValues[2] = trigger;
		m_compArray = this.initUserInterface();
	}
	
	public MacdParameters(String indParameters) {
		m_parameterValues = this.fromParameterString(indParameters);
		m_compArray = this.initUserInterface();
	}

	public JComponent[][] getUserInterface() {
		if (m_compArray == null) {
			m_compArray = this.initUserInterface();
		}
		return m_compArray;
	}

	public void updateParameters() {
		this.setParameters(0, Integer.parseInt(macdFastText.getText()));
		this.setParameters(1, Integer.parseInt(macdSlowText.getText()));
		this.setParameters(2, Integer.parseInt(macdTriggerText.getText()));            
	}

	public void setParameters(int index, int value) {
		// Set a resonable default
		if (value < 0)  {
			if (index == 0) {
				// Fast default
				value = 12;
			} else if (index == 1) {
				// Slow default
				value = 26;
			} else if (index == 2) {
				// Trigger default
				value = 9;
			}
		} 
		m_parameterValues[index] = value;
		if (m_compArray[index][1] != null) {
			// Update JTextField with new values
			JTextField tempField = (JTextField) m_compArray[index][1];
			tempField.setText(String.valueOf(value));
		}
//		if (macdText != null) {
//			macdText.setText(String.valueOf(value));
//		}
	}

	public void setParameters(int [] parameters) {
		m_parameterValues = parameters;
		for (int index = 0; index < m_compArray.length; index ++) {
			if (m_compArray[index][1] != null) {
				// Update JTextField with new values
				JTextField tempField = (JTextField) m_compArray[index][1];
				tempField.setText(String.valueOf(parameters[index]));
			}
		}
//		if (macdText != null) {
//			macdText.setText(String.valueOf(parameters[0]));
//		}
	}
	
	public String [] getParameterNames() {
		String [] strA = new String [3];
		strA[0] = "MACD.Fast";
		strA[1] = "MACD.Slow";
		strA[2] = "MACD.Trigger";
		return strA;
	}
	
	public boolean indicatorEnabled() {
		return this.parametersSet();
	}
	

	/**
	 * Does not clone the compArray
	 */
	public Object clone() {
	   	MacdParameters o = null;
        try {
        	o = (MacdParameters)super.clone();
        } catch(CloneNotSupportedException e) {
        	logger.error(e.toString());
        }
        o.m_parameterValues = (int []) o.m_parameterValues.clone();
        return o;	
        
//		MacdParameters clone = new MacdParameters(
//				this.m_macdParameters[0], this.m_macdParameters[1], this.m_macdParameters[2]);
//		return clone;
	}
	
	private JComponent[][] initUserInterface() {
		//JComponent [][] compArray;

		JLabel macdFast = new JLabel("MACD Fast");
		macdFastText = new JTextField(3);
		macdFastText.setText(String.valueOf(this.m_parameterValues[0]));

		String macdFastTip = "Enter the number of days to use as fast EMA in MACD calculation. Use 0 for no MACD.";
		macdFastText.setToolTipText(macdFastTip);
		macdFast.setToolTipText(macdFastTip);

		JLabel macdSlow = new JLabel("MACD Slow");
		macdSlowText = new JTextField(3);
		macdSlowText.setText(String.valueOf(this.m_parameterValues[1]));

		String macdSlowTip = "Enter the number of days to use as slow EMA in MACD calculation. Use 0 for no MACD.";
		macdSlowText.setToolTipText(macdSlowTip);
		macdSlow.setToolTipText(macdSlowTip);
		
		JLabel macdTrigger = new JLabel("MACD Trigger");
		macdTriggerText = new JTextField(3);
		macdTriggerText.setText(String.valueOf(this.m_parameterValues[2]));

		String macdTriggerTip = "Enter the number of days to use as Trigger/Signal line in MACD calculation. Use 0 for no MACD.";
		macdTriggerText.setToolTipText(macdTriggerTip);
		macdTrigger.setToolTipText(macdTriggerTip);


		m_compArray = new JComponent[3][];
		m_compArray[0] = new JComponent[2]; 
		m_compArray[1] = new JComponent[2]; 
		m_compArray[2] = new JComponent[2]; 

		m_compArray[0][0] = macdFast;
		m_compArray[0][1] = macdFastText;
		m_compArray[1][0] = macdSlow;
		m_compArray[1][1] = macdSlowText;
		m_compArray[2][0] = macdTrigger;
		m_compArray[2][1] = macdTriggerText;

		return m_compArray;
	}

	private boolean parametersSet() {
		boolean result = true;
		for (int index = 0; index < m_parameterValues.length; index ++) {
			// Check that all parameters are set
			if (m_parameterValues[index] <= 0) {
				result = false;
			}
		}
		return result;
	}

	public String toParameterString() {
		return  this.toString();
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorParametersInterface#fromParameterString()
	 */
	public int [] fromParameterString(String s) {
		int [] result = new int[3];
		String [] temp = s.split(sm_delimiter);
		for (int i = 0; i < result.length; i ++) {
			try {
				result[i] = Integer.parseInt(temp[i]);	
			} catch (Exception e) {
				logger.error(e.toString());
			}
		}
		return result;
	}

//	public int getNumberOfDays() {
//	return numberOfDays[0];
//}

///**
// * @param i the number of days used in the RSI calculation
// */
//public void setNumberOfDays(int i) {
//	numberOfDays[0] = i;
//	if (macdText != null) {
//		macdText.setText(String.valueOf(i));
//	}
//}	
//	public boolean equals (Object o) {
//	MacdParameters otherMacd = (MacdParameters) o;
//	if (otherMacd == null) return false;
//	boolean result = true;
//	int [] otherMacdParameters = otherMacd.getParameters();
//	for (int index = 0; index < this.macdParameters.length; index ++) {
//		// Check if equal
//		if (this.macdParameters[index] != otherMacdParameters[index]) {
//			result = false;
//		}
//	}
//	return result;
//}


//public String toString() {
//	String s ="";
//	for (int index = 0; index < this.macdParameters.length; index ++) {
//		if (!s.equalsIgnoreCase("")) {
//			s = s + delimiter + this.macdParameters[index];
//		} else {
//			s =  s + this.macdParameters[index];
//		}
//	}
//	return s; 
//}

}
