/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 08-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.util.ArrayList;
import java.text.DecimalFormat;

import com.exponto.jstock.gui.panels.GraphicsParameters;

/**
 * 
 * 
 * @author kaare
 *
 */
public class Rsi extends Indicator implements IndicatorValuesInterface, IndicatorLinesInterface {
	private static SingleParameterParameters m_defaultParameters;
	private SingleParameterParameters m_parameters;
	private static final String toolTip = "Enter the number of days to use in Relative Strength Index calculation. Use 0 for no RSI.";
	GraphicsParameters m_graphicsParameters;
	IndicatorAxisInfo m_indicatorAxisInformation = null;
	protected ArrayList m_indicatorSeries;
	
	int m_oldNumberOfDays = 0;
	//boolean m_parametersChanged = true;
	int m_widthOfDay;
	int m_numberOfDaysToDisplay;
	double m_scaleY = 0.01;
	int m_width;
	int m_height;
	int m_x0, m_y0;
	private static final String propName = "RSI.NumberOfDays";  
	
	public Rsi () {
		super();
		sm_indicatorName = Constants.rsi;
		m_indicatorSeries = new ArrayList();
		if (m_defaultParameters == null) {
			m_defaultParameters = new SingleParameterParameters(14, propName, sm_indicatorName, toolTip);
		}
		sm_indicatorColor = Color.red; 
	}

	public IndicatorParameters getIndicatorParameters() {
		if (m_parameters == null) {
			return m_defaultParameters;
		} 	
		return m_parameters;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(java.lang.String, com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		if (m_parameters == null) {
			if (m_oldNumberOfDays != m_defaultParameters.getNumberOfDays()) {
				// Only update if parameters has changed
				m_indicatorSeries = loadRsi(lDS, m_defaultParameters);	
			}
			//m_parametersChanged = true;
			//logger.debug("Default indicatorSeries for RSI used");
		} else {
			if (m_oldNumberOfDays != m_parameters.getNumberOfDays()) {
				// Only update if parameters has changed
				m_indicatorSeries = loadRsi(lDS, m_parameters);	
			}
			//m_parametersChanged = true;
			//logger.debug("Specific indicatorSeries for RSI used");
		}
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.Indicator#initiateIndicator(com.exponto.jstock.LogDataSeries, java.lang.String)
	 */
	public void initiateIndicator(LogDataSeries lDS, String indParam) {
		SingleParameterParameters tempRsiParameters = new SingleParameterParameters(indParam, propName, sm_indicatorName, toolTip);
		if (m_oldNumberOfDays != tempRsiParameters.getNumberOfDays() || m_parameters == null){
			// Uses m_rsiParameters if non existent and updates if parameters has changed
			m_parameters = tempRsiParameters;
			m_indicatorSeries = loadRsi(lDS, m_parameters);
			//m_parametersChanged = true;
			logger.debug("indicatorSeries for RSI has changed");
		}
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return false;
	}
	 public boolean hasChanged() {
	 	boolean result = true;
	 	if (m_parameters == null) {
	 		if (m_oldNumberOfDays == m_defaultParameters.getNumberOfDays()) {
	 			result = false;
	 		}
	 	} else {
	 		if (m_oldNumberOfDays == m_parameters.getNumberOfDays()) {
	 			result = false;
	 		}
	 	}
	 	return result;
	 }
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D)
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP) {
		m_graphicsParameters = gP;
		//System.out.println("RSI size is " + m_graphicsParameters.getDimension());
		m_width =  gP.getNetWidth();
		m_height = (int) gP.getDimension().getHeight() - gP.getTopMargin() - gP.getBottomMargin();
		m_x0 = gP.getLeftMargin();
		m_y0 = m_height + gP.getTopMargin();
		m_scaleY = (double) m_height/100;
		//logger.debug("RsiPanel - height, scaleY, y0: " + height + ", " + scaleY + ", " + y0);
		m_widthOfDay =gP.getPixelPrDay();
		m_numberOfDaysToDisplay = m_width / m_widthOfDay;
		//g2d.setBackground(Color.lightGray);
		g2d.setColor(Color.black);
		//drawTitle(g2d);
		drawAxis(g2d);
		if (m_indicatorSeries == null) {
			logger.error("indicatorSeries for RSI is null");
		} else if (m_indicatorSeries.size() == 0){
			logger.error("indicatorSeries for RSI has size of null");
		} else {
			drawRSI(g2d);			
		}
		// Update status variables
		//m_parametersChanged = false;
		if (m_parameters == null) {
			m_oldNumberOfDays = m_defaultParameters.getNumberOfDays();	
		} else {
			m_oldNumberOfDays = m_parameters.getNumberOfDays();
		}
		//logger.debug("RSI panel repainted");
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		String s = " = ";
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(2);
		df.setDecimalSeparatorAlwaysShown(false);
		String value ="NA";
		int index = 0;
		if (n < 0) {
			index = m_indicatorSeries.size() -1;
			s = ", last value is ";
		} else {
			//System.out.println("index: " + index);
			int displayWidth = m_graphicsParameters.getNetWidth()/ m_graphicsParameters.getPixelPrDay();
			if (m_indicatorSeries.size() >= displayWidth) {
				// Series is longer than display width
				index = m_indicatorSeries.size() -1 - (displayWidth);
			}
			//System.out.println("index I: " + index + " Series, display  length: " + indicatorSeries.size() + ", " + displayWidth);
			index = n + index;
		}
		if ((index < m_indicatorSeries.size()) && (index >= 0)) {
			Double  d = (Double) m_indicatorSeries.get(index);
			if (d != null) {
				value = df.format(d);
			}
		}
		//System.out.println("index II: " + index);
		return this.toString() + s + value;
	}

	public String toString() {
		if (m_parameters == null) {
			return sm_indicatorName +"(" + m_defaultParameters.getNumberOfDays() + ")";
		}
		return sm_indicatorName +"(" + m_parameters.getNumberOfDays() + ")";
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int, int)
	 */
	public String[] printAll(int number) {
		Double d;
		String [] sA = new String[number + 1];
		int start = m_indicatorSeries.size() - number;
		int index = 1;
		sA[0] = this.toString();
		if (start < 0) start = 0;
		for (int i = start; i < m_indicatorSeries.size(); i++) {
			d = (Double) m_indicatorSeries.get(i);
			if (d != null) {
				sA[index++] = d.toString();
			} else {
				sA[index++] = "empty";
			}
		}
		return sA;
	}

	public IndicatorAxisInfo getIndicatorAxisInfo() {
		if (m_indicatorAxisInformation== null) {
			// For RSI the y axis is linear from 0 to 100
			m_indicatorAxisInformation = new IndicatorAxisInfo(0, 100, Constants.linear);
		}
		return m_indicatorAxisInformation;
	}
	
	public boolean supportIndicatorValuesInterface() {
		return true;
	}

// Public methodes below is for IndicatorValuesInterface
	public String[] parameters() {
		String[] strA = new String[1];
		// Might be more appropriate to not use toString() as the number of days is unneeded.
		strA[0] = this.toString();
		return strA;
	}
	
	public String[] parametersToolTip() {
		String[] strA = new String[1];
		strA[0] = "The value of RSI.";
		return strA;
	}
	
	public Double parameterValue(String str) {
		// No need to check string as there is only one parameter
		Double value;
		value = (Double) m_indicatorSeries.get(m_indicatorSeries.size() - 1);
		return value;
	}

	public Double parameterValue(String str, int index) {
		// No need to check string as there is only one parameter
		Double value;
		int i = m_indicatorSeries.size() - 1 - index;
		if (i < 0) {
			value = null;
		} else {
			value = (Double) m_indicatorSeries.get(i);	
		}
		return value;
	}

	private void drawAxis(final Graphics2D g2d) {
		int y;
		int limitHi = 80;
		int limitLow = 20;
		//x-axis
		Line2D line = new Line2D.Double (m_x0, m_y0, m_x0 + m_width, m_y0);
		g2d.draw(line);
		//y-axis
		line.setLine(m_x0, m_y0, m_x0, m_y0 - m_height);
		g2d.draw(line);
		// Lower limit line & text
		g2d.setColor(Color.gray);
		y = m_y0  - (int) (limitLow * m_scaleY) ;
		line.setLine(m_x0, y, m_x0+ m_width, y);
		g2d.draw(line);
		String str = String.valueOf(limitLow);
		g2d.drawString(str, 15, y+5);
//		High limit line & text
		y = m_y0  - (int) (limitHi * m_scaleY) ;
		line.setLine(m_x0, y, m_x0+ m_width, y);
		g2d.draw(line);
		str = String.valueOf(limitHi);
		g2d.drawString(str, 15, y+5);
	}

	private void drawRSI(final Graphics2D g2d) {
		Double value;
		double y = 0.0, y2 = 0.0;
		int length;
		int x = m_x0 + m_widthOfDay/2;
		Line2D line = new Line2D.Double ();
		g2d.setColor(sm_indicatorColor);
//		logger.debug("RsiPanel - ArraySize, Days to display: " + rsiList.size() + ", " + numberOfDaysToDisplay);
		length = m_indicatorSeries.size() - m_numberOfDaysToDisplay - 1;
//		Ensure that i is within array bounds
		if (length < 0) {
			length = 0;
			logger.debug("Length is less than zero");
		}
		value = (Double) m_indicatorSeries.get(length ++);
		// Only null at the begining of the series (famous last words...)
		while (value == null) {
			value = (Double) m_indicatorSeries.get(length ++);
			x = x + m_widthOfDay;
		}
		y = m_y0  - value.doubleValue() * m_scaleY ;
		for (int i = length; i < m_indicatorSeries.size(); i++) {
			// Ensure that i is within array bounds
			value = (Double) m_indicatorSeries.get(i);
			y2 = m_y0  - value.doubleValue() * m_scaleY ;
			line.setLine(x, y, x + m_widthOfDay, y2);
			g2d.draw(line);
			//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
			x = x + m_widthOfDay;
			y = y2;
		}
		//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
	}

	private ArrayList loadRsi(LogDataSeries lDS, SingleParameterParameters rsiParam) {
		//m_parametersChanged = true;
		return RsiAlgorithm.findRSIClassicWay(lDS, rsiParam);
	}

//	private void drawTitle(final Graphics2D g2d) {
//	// Title
//	String str = "RSI("+String.valueOf(rsiParameters.getNumberOfDays())+")";
//	g2d.drawString(str, 5, (int) y0 - height);
//}
	
//	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
//	// Is currently not used, and is not working very well
//	if (m_oldNumberOfDays != m_defaultRsiParameters.getNumberOfDays()){
//		// 
//		m_indicatorSeries = loadRsi(lDS, (SingleParameterParameters) indParam);
//		m_parametersChanged = true;
//		logger.debug("indicatorSeries for RSI has changed");
//	}
//}

}
