/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Contains Moving Average specific parameters used in computation of an
 * indicator series. It may also be used by related indicators using the same 
 * kind of parameters. Parameter name (for properties file) may be specified
 * in the Constructor.
 * 
 * @author kaare
 *
 */
public class SingleParameterParameters extends IndicatorParameters {
	JTextField m_textField;
	String propertyName;
	String toolTip = "Enter the number of days to use in Moving Average calculation. Use 0 for no MA.";
	String labelName = "MA";

	public SingleParameterParameters(String s, String propName, String label, String toolTipString) {
		m_parameterValues = new int[1];
		m_parameterValues= this.fromParameterString(s);
		propertyName ="propName";
		labelName = label;
		toolTip = toolTipString;
	}

	
	public SingleParameterParameters(int days, String propName, String label, String toolTipString) {
		m_parameterValues = new int[1];
		m_parameterValues[0] = days;
		propertyName =propName;
		labelName = label;
		toolTip = toolTipString;
	}

	public String [] getParameterNames() {
		String [] strA = new String [1];
		strA[0] = propertyName;
		return strA;
	}

	/**
	 * @return the number of days used in the calculation
	 */
	public int getNumberOfDays() {
		return m_parameterValues[0];
	}

	/**
	 * @param i the number of days used in the calculation
	 */
	public void setNumberOfDays(int i) {
		m_parameterValues[0] = i;
	}

	public JComponent[][] getUserInterface() {
		if (m_compArray == null) {
			m_compArray = this.initUserInterface();
		}
		return m_compArray;
	}

	public boolean indicatorEnabled() {
		boolean result = false;
		if (this.getNumberOfDays() > 0) {
			result = true;
		} 
		return result;
	}

	public void updateParameters() {
		try {
			this.setParameters(0, Integer.parseInt(m_textField.getText()));            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read input for MA");
		}
	}

	private JComponent[][] initUserInterface() {
		//JComponent [][] compArray;
		JLabel ma = new JLabel(labelName);
		m_textField = new JTextField(3);
		m_textField.setText(String.valueOf(this.getNumberOfDays()));
		m_textField.setToolTipText(toolTip);
		ma.setToolTipText(toolTip);

		m_compArray = new JComponent[1][];
		m_compArray[0] = new JComponent[2]; 
		m_compArray[0][0] = ma;
		m_compArray[0][1] = m_textField;
		return m_compArray;
	}

	public String toParameterString() {
		return  String.valueOf(m_parameterValues[0]);
	}


	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorParametersInterface#fromParameterString()
	 */
	public int [] fromParameterString(String s) {
		int [] result = new int[1];
		result[0] = Integer.parseInt(s);
		return result;
	}
	
	public int hashCode() {
		return m_parameterValues[0];
	}
	/**
	 * Only checks that the Indicator parameters are equal. Does not care about
	 * tooltips etc.
	 */
	public boolean equals(Object o) {
		boolean result = false;
		SingleParameterParameters temp = (SingleParameterParameters) o;
		if (this.getNumberOfDays() == temp.getNumberOfDays()) {
			result = true;
		}
		return result;
	}
	
//	public SingleParameterParameters() {
//	numberOfDays[0] = 14;
//	propertyName ="MA.NumberOfDays";
//}


//public SingleParameterParameters(int days) {
//	numberOfDays[0] = days;
//	propertyName ="MA.NumberOfDays";
//}
}
