/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */



/*
 * Created on 06-Oct-2003
 *
 */
package com.exponto.jstock.io;
import java.sql.*;
import org.apache.log4j.Logger;


/**
 * A manager class for the database. Handles opening and closing 
 * of connections and simple updates and inserts.
 * 
 * This is a singleton.
 *  
 * @author kaare
 *
*/
public class DBManager {
	static String dbdriver;
	String user;
	String pw;
	String host;
	String dbName;
	String URL;
	
	static Logger logger = Logger.getLogger(DBManager.class);
	Connection con = null;
	static DBManager instance;
	UserPreferences userPref;
	
	private DBManager() {
		userPref = UserPreferences.getInstance();
		dbdriver = userPref.getJdbcDriver();
		user = userPref.getDbUser();
		host = userPref.getDbHost();
		pw = userPref.getDbPW();
		dbName = userPref.getDbName();
		try {
			Class.forName(dbdriver);
			//System.out.println("Class DBManager: Driver found");
		} catch (Exception e) {
			logger.error("Class DBManager: Could not find driver for DB " + dbdriver);
			//System.out.println("Class DBManager: Could not find driver for DB " + dbdriver);	
		}
		URL ="jdbc:postgresql://" + host + "/" + dbName + "?user=" + user + "?password=" + pw;
	}	
	/**
	 * Use this one to get an instance of this class. Will create one if it 
	 * doesn't exist.
	 * 
	 * @return the instance of the class
	 */
	static public DBManager getInstance() {
		if (instance==null) {
			instance = new DBManager();
		}
		return instance;
	} 
/**
 * Use this to get a db connection. Logfile is updated in case of error.
 * 
 * @return a db Connection if successful, otherwise null. 
 */		
	public Connection getConnection() {
		if (con == null) {
			try {
				con = DriverManager.getConnection(URL);
			} catch (Exception e) {
				logger.error("Class DBManager: Could not create connection to" + URL);
				logger.error(e);
				return null;
				//System.out.println("Class DBManager: Could not create connection to" + URL);
				//System.out.println(e);
			}
		}
		return con;
	}
/**
 * Closes the db connection
 *
 */	
	public void closeConnection() {
		if (con != null) {
			try {
				con.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		logger.info("Class DBManager: DBConnection closed");
		//System.out.println("Class DBManager: DBConnection closed");
	} 
	
/**
 * Executes simple updates or inserts. 
 * 
 * In case of error, 0 is returned and additional information is in the logfile.  
 * 
 * @param sql - the complete sql statement that is to be executed. 
 * @return the number of rows that was affected by the update
 */
	public static int executeUpdate(String sql) {
		int rowCount;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();		
		try {
			Connection connection = dBMan.getConnection();
			sqlStmnt = connection.createStatement();
			rowCount = sqlStmnt.executeUpdate(sql);
		} catch (Exception e) {
			logger.error("Class DBManager: Could not execute " + sql);
			logger.error(e.toString());
			//System.out.println("Class DBManager: Could not execute " + sql);
			e.printStackTrace();
			return 0;
		}
		return rowCount;
	}
}
