/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package com.exponto.jstock.io;

/**
 * Reads a generic file containg quotes for a stock. The position of the
 * fields may be controlled by the user
 *  
 * @author  kaare
 */
import java.io.* ;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Calendar;
import org.apache.log4j.Logger;

import com.exponto.jstock.exception.TickerNotFoundException;
import com.exponto.jstock.indicators.LogDataSeries;
import com.exponto.jstock.indicators.LogTradingData;
import com.exponto.util.*;

public class ReadGenericDataFile {
	static Logger logger = Logger.getLogger(ReadGenericDataFile.class);
	private boolean eof = false;
	private BufferedReader reader = null; 
	private String fileName = null;
	private static UserPreferences uP;
	GregorianCalendar cal = new GregorianCalendar();
    
    double open = 0.0;
	double high = 0.0;
	double low = 0.0;
	double close = 0.0;
	long volume = 0L;
	String ticker;
//	String exchange;
	//String s;
    /** Creates a new instance of ReadGenericFile */
	public ReadGenericDataFile()  {
		eof = false;
	}
    
     /**
     * Read data from file and store the result. 
     * This one uses the column positions from the UserPreferences.
     * 
     * Column separators can be '\t',' ', ',' (that is tab, space or comma).
     * 
     * Also, in order for the to separate column headers from the data
     * it is required that data lines start with a digit. 
     * 
     * Curently files can only be read from begin to end, that means it
     * assumes that latest data is at the beginning. (Doesn't matter if
     * whole file is read (numberToGet is larger than number of lines 
     * in file.
     * 
     */   
    public synchronized LogDataSeries readSeries (String fN, int numberToGet) throws TickerNotFoundException {
    	uP = UserPreferences.getInstance();
    	SimpleDateFormat sDF  = new SimpleDateFormat(uP.getDateFormat());
    	Date day = null; 
    	
    	fileName = fN;
    	int i = 0;
    	LogTradingData lTD;
        LogDataSeries lDS = new LogDataSeries();
        //Character ch = new Character('e');
	    try {
			reader = new BufferedReader (new FileReader (fileName));
            while (true) {
            	i ++;
                String str = this.readString();
				if (i == numberToGet) return lDS;
                if (eof) return lDS;
                // Check if string (may) contain valid data, i.e. longer than 7 
                // characters and that the first character is a digit
                // Is it possible to avoid digit check, e.g. accept a few exceptions in the beginning?
                if ((str.length() > 7) && Character.isDigit(str.charAt(0))) {
                	// Start with date field
                	try {
						cal.setTime(sDF.parse((Util.findSubString(str, uP.getDatePos()))));
						day = new Date(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));
						day.findWeekDay();
                	} catch (Exception e) {
                		logger.error(e.toString());
						logger.error("NOTE: The parse methode of SimpleDateFormat can be picky, make sure that the date format is correct in the OPTIONS/File input menu");               		
                	}
                	if (uP.getClosePos() >= 0) {
                		close = findDouble(Util.findSubString(str, uP.getClosePos()));
                	} else {
                		logger.error("Could not find any closing quote, this is the only quote that really is needed");
                		// Throw some exception
                	}
                	if (uP.getOpenPos() >= 0) {
						open = findDouble(Util.findSubString(str, uP.getOpenPos()));
                	} else {
                		open = close;
                	}
					if (uP.getHighPos() >= 0) {
						high = findDouble(Util.findSubString(str, uP.getHighPos()));
					} else {
						high = close;
					}
					if (uP.getLowPos() >= 0) {
						low = findDouble(Util.findSubString(str, uP.getLowPos()));
					} else {
						low = close;
					}
					if (uP.getVolumePos() >= 0) {
						volume = findLong(Util.findSubString(str, uP.getVolumePos()));
					} else {
						volume = 0;
					} 
					// Create the data set
					lTD = new LogTradingData(day, open, high, low, close, volume, Constants.unknown);					
//                	// A very Q&D way of getting ticker and exchange in lDS
//                	if (i == 8) {
//						if (uP.getTickerPos() >= 0) {
//							lDS.setTicker(Util.findSubString(str, uP.getTickerPos()));
//						} else {
////							TO DO: Add ticker and exchange even if not present in file
//							logger.debug(" File name is: " + fN);
//						}
//						if (uP.getExchangePos() >= 0) {
//							lDS.setExchange(Util.findSubString(str, uP.getExchangePos()));
//						} 
//                	}
                	lTD.setStatus();
                    lDS.storeResult(lTD);
                    //System.out.println(lTD.toString());
                }
            }
        } 
        catch (FileNotFoundException e) {
        	throw new TickerNotFoundException (" Could not find requested ticker in file " + fileName);
            //System.out.println(" Fant ingen " + fileName + ", er det noe som ikke stemmer?");
        }
//        return null;
    }
    
    /**
     * Reads one line of the quotes file, hopefully that is data for one 
     * single day.
     * @return a string with the line.
     */
	public final synchronized String readString() {
		String s = "";
		try {
			s = reader.readLine();
	  	}
	  	catch (IOException e)  {
	  		logger.error("IOException, Failed to read string for file" + fileName);
	  		logger.error(e.toString());
			//System.out.println("Klasse ReadNetfondsFile, readString feilet for filen: " + fileName);
	  }
	  if (s == null) {
	  		eof = true;
			s = "";
	  }
	  return s;
	}
	
	private double findDouble(String s) {
		double d = -1.0;
		try {
			d = Double.parseDouble(s) ;
		} catch (NumberFormatException e) {
			logger.error(e.toString());
			logger.error("Unable to convert a string to double, check if the column numbers are correct for open, high, low or close");
		}
		return d;
	}

	private long findLong(String s) {
		long l = -1L;
		try {
			l = Long.parseLong(s) ;
		} catch (NumberFormatException e) {
			logger.error(e.toString());
			logger.error("Unable to convert a string to long, check if the column numbers are correct for volume");
		}
		return l;
	}
    
}
