/*
 *                                 jStock
 *                    (C) Copyright 2003, 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 11-Oct-2003
 *
 */
package com.exponto.jstock.io;

/**
 * Container that uniquly identifies a ticker.
 * @author kaare
 *
 */
public class TickerID {
	private int m_tickerIndex;
	private String m_tickerName;
	private String m_exchangeName;

	/**
	 * 
	 */
	public TickerID() {
		
	}
/**
 * Creates a new TickerID. Both Strings are converted to lower case. 
 * Note that the input parameters are copied, so this one
 * should be safe. 
 * 
 * @param tN the ticker
 * @param eN the exchange
 */
	public TickerID(String tN, String eN) {
		m_tickerName =new String(tN).toLowerCase();
		m_exchangeName =new String(eN).toLowerCase();
	}
	
	public TickerID(int t, String tN, String eN) {
		m_tickerIndex = t;
		m_tickerName = tN;
		m_exchangeName = eN;
	}

	/**
	 * @return the exchange name
	 */
	public String getExchangeName() {
		return m_exchangeName;
	}

	/**
	 * @return the Identity of the ticker as used by the database. 
	 * The naming might be a bit confusing... 
	 */
	public int getTickerIndex() {
		return m_tickerIndex;
	}

	/**
	 * @return the ticker name
	 */
	public String getTickerName() {
		return m_tickerName;
	}

	/**
	 * @param string the exchange name
	 */
	public void setExchangeName(String string) {
		m_exchangeName = string.toLowerCase();
	}

	/**
	 * @param i the ticker identity as used by the database
	 */
	public void setTickerIndex(int i) {
		m_tickerIndex = i;
	}

	/**
	 * @param string the ticker name
	 */
	public void setTickerName(String string) {
		m_tickerName = string.toLowerCase();
	}
	
	public String toString() {
		return m_tickerName + "." +  m_exchangeName;
	}

	public String toDebugString() {
		String s = "TickerId: " + String.valueOf(m_tickerIndex) + " " + m_tickerName + "." +  m_exchangeName;
		return s;
	}
	
	public int hashCode() {
		int result = 211;
		result = result + m_tickerName.hashCode() + m_exchangeName.hashCode() + m_tickerIndex;
		return result;
	}
	
	public boolean equals(Object o) {
		boolean result = true;
		TickerID tId = (TickerID) o;
		if (this.m_tickerIndex != tId.getTickerIndex()) {
			result = false;
		}
		if (!this.m_tickerName.equals(tId.getTickerName())) {
			result = false;
		}
		if (!this.m_exchangeName.equals(tId.getExchangeName())) {
			result = false;
		}
		return result;
	}
}
