/*
 *                                 jStock
 *                    (C) Copyright 2003, 2004 Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 31-Oct-2003
 *
 */
package com.exponto.jstock.io;


import com.exponto.jstock.indicators.LogDataSeries;
import com.exponto.jstock.indicators.LogTradingData;
import com.exponto.util.*;
//import com.exponto.util.*;

/**
 * Some static methods that are more or less usefull.
 * @author kaare
 *
 */
public abstract class Util {
	/**
	 * Converts a string to LogTradingData
	 * 
	 * @param str that includes all needed data for a day
	 * @return Logtradingdata for the day
	 */
	public static LogTradingData convertString(String str) {
		LogTradingData lTD = parseString(str);
		lTD.status = validate(lTD);
		return lTD;
	}

	public static LogTradingData convertYahooString(String str) {
		LogTradingData lTD = parseYahooString(str);
		lTD.status = validate(lTD);
		return lTD;
	}

	
	/**
	 * Validate that the given combination of open, high, low, close and volume 
	 * values are likely to correct.
	 * 
	 * Tests and results:
	 * 	If all zero -> assumes that stock is not traded
	 * 	volume zero, but others not -> faulty
	 *		low > high -> faulty
	 *		!high >= open >= low -> faulty
	 *		!high >= close >= low -> faulty
	 *		if high > 3* low -> drop
	 *		otherwise -> ok
	 *
	 * Note that data will be marked as "drop" if high > 3 * low
	 *
	 * If only open and close is available either set high and low to the
	 * higher/lower of opne /close or modify the tests
	 * 
	 * @author  kaare 
	 * @version 0.5
	 * @changes
	 *
	 * @param LogTradingData	Data to be verified
	 * @return char	Result of verification
	 */ 
	public static char validate(LogTradingData lTD) {
		char status = Constants.unknown;
		// To verify that the stock was (should be) traded that day
		if (lTD.volume > 0) {
			// Check if any fields are zero or less
// Note that this would cause datasets where only open and close are 
// known to fail, either reimplement or set high/low to same as 
// high/low  of open close
			if (lTD.open <= 0 || lTD.low <= 0 || lTD.close <= 0 || lTD.high <= 0) {
				status = Constants.faulty;
				//return status;
//				Check if low is larger then high
			} else if (lTD.high < lTD.low) {
				status = Constants.faulty;
				//return status;
			} else if (lTD.open > lTD.high || lTD.open < lTD.low) {
				status = Constants.faulty;
				//return status;
			} else if (lTD.close > lTD.high || lTD.close < lTD.low) {
				status = Constants.faulty;
				//return status;
			}
			if (lTD.high/lTD.low > 3)  {
				// Ignore if stock changes more than 300% on a single day 
				status = Constants.drop;
				//return status;
			} else {
				// Everything assumed to be OK
				status = Constants.ok;
				//return status;
			}
			//return status;
			// System.out.println(" minkurs. "+ minkurs + " lav: " + lav );
		} else {
			if (lTD.open == 0 && lTD.low == 0 && lTD.close == 0 && lTD.high == 0) {
				status = Constants.noTrade;
				//return status;
			} else {
				status = Constants.faulty;
				//return status;
			}
			//return status;
		}
		return status;
	}
	/**
	 * Creates a new LogTradingData and puts the content of the string
	 * into the LogTradingData. The data is not validated. If the string
	 * contains non valid content null is returned
	 * 
	 *
	 * @author  kaare 
	 * @version 0.5
	 * @changes
	 *
	 * @param a string with all quote information for a single day.
	 * @return a LogTradingData with the same information.
	 */ 	
	private static LogTradingData parseString(String datastr) {
		LogTradingData lTD = new  LogTradingData();
		int position = 0;
		int element = 0;
		char ch  = ' ';
//		Character cha = new Character('e');
		String str = "";
		String [] strarray = { "", "", "", "", "", "", ""};
		while (position < datastr.length()) {
			ch = datastr.charAt(position);
			if (ch == '\t'||ch == ' ') {
				// Only keeps values that start with a digit,'.' or ','
				if (Character.isDigit(str.charAt(0)) ||str.charAt(0) =='.' || str.charAt(0) == ',' ) {
					strarray[element] = str;
					element ++;
				} else if (str.charAt(0) =='-') {
					// '-' is sometimes used probably for unknown 
					strarray[element] = "0";
					element ++;
				}
				str ="";
			}  else {
				str = str + String.valueOf(ch);
			}
			position ++;
		}
		strarray[element] = str;
		lTD.day.convert(strarray[0]);
		try {
			lTD.open = Double.parseDouble(strarray[1]) ;
			lTD.high = Double.parseDouble(strarray[2]);
			lTD.low = Double.parseDouble(strarray[3]);
			lTD.close = Double.parseDouble(strarray[4]);
			lTD.volume = Long.parseLong(strarray[5]) ;            
		} catch (NumberFormatException e) {
			System.out.println("Class Util, method parseString: NumberFormatException in " + datastr);
			return null;
		}
		lTD.status = Constants.unknown;
		return lTD;
	}
	
	/**
	 * Apart from different date converter this is equal to parseString, should be 
	 * joined at some time.
	 * @param datastr
	 * @return
	 */
	private static LogTradingData parseYahooString(String datastr) {
		LogTradingData lTD = new  LogTradingData();
		int position = 0;
		int element = 0;
		char ch  = ' ';
//		Character cha = new Character('e');
		String str = "";
		String [] strarray = { "", "", "", "", "", "", ""};
		while (position < datastr.length()) {
			ch = datastr.charAt(position);
			// tab or comma as separator
			if (ch == '\t'||ch == ',') {
				// Only keeps values that start with a digit or '.' 
				if (Character.isDigit(str.charAt(0)) ||str.charAt(0) =='.' ) {
					strarray[element] = str;
					element ++;
				} else if (str.charAt(0) =='-') {
					// '-' is sometimes used probably for unknown 
					strarray[element] = "0";
					element ++;
				}
				str ="";
			}  else {
				str = str + String.valueOf(ch);
			}
			position ++;
		}
		strarray[element] = str;
		lTD.day.convertYahoo(strarray[0]);
		try {
			lTD.open = Double.parseDouble(strarray[1]) ;
			lTD.high = Double.parseDouble(strarray[2]);
			lTD.low = Double.parseDouble(strarray[3]);
			lTD.close = Double.parseDouble(strarray[4]);
			lTD.volume = Long.parseLong(strarray[5]) ;            
		} catch (NumberFormatException e) {
			System.out.println("Class Util, method parseString: NumberFormatException in " + datastr);
			return null;
		}
		lTD.status = Constants.unknown;
		return lTD;
	}
	
/** 
 * Validates that a data series is likly to be correct 
 * @param lDS The LogDataSeries to be validated
 * @return true if the data MIGHT be corect
 */
	public static boolean validate(LogDataSeries lDS) {
		boolean valid = true;
		LogTradingData lTD;
		lDS.setCurrentPeriode();
		lTD = lDS.getNextLTD();
		Date currentDate = lTD.day.klone();
		//System.out.println("Class Util, validate: Starting at: " + currentDate.toString());
		while (lTD != null) {
			if (lTD.day.compareDate(currentDate) != 0) {
				valid = false;
				//System.out.println("Class Util, validate: Expected date was: " + currentDate.toString() + " while found date was: " + lTD.day.toString());
			} else if (lTD.isTraded()) {
				if (lTD.low == 0) {
					//System.out.println("Class Util, validate: Traded low was 0 at date: " + lTD.day.toString());
					valid = false;
				}
			}
			currentDate.setToNextWorkDay();
			lTD = lDS.getNextLTD();
		}
		//System.out.println("Class Util, validate: Next expected day is: " + currentDate.toString());
		return valid;	
	}

	/** 
	 * Find a substring in tab, comma or space separated string. 0 is first substring 
	 * 
	 * @param String to search
	 * @param which substring is to be returned
	 * @return  the substring that is found.
	 */	
	public static String findSubString(String s, int i) {
		int position = 0;
		int element = 0;
		char ch  = ' ';
		String str = "";
		while (position < s.length()) {
			ch = s.charAt(position);
			// Delimiters can be tab, space or comma 
			if (ch == '\t'||ch == ' '|| ch ==',') {
				if (element == i) {
					// Found what we are looking for
					return str;
				}
				// Clear and try next substring
				element ++;
				str ="";
			} else {
				str = str + String.valueOf(ch);
			}
			position ++;
		}
		return null;	
	}

	public static boolean evalute(Object[] rule, LogTradingData lTD) {
		// TODO: Write evaluation
		return true;
	}
}
