/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 27-Jun-2004
 *
 */
package com.exponto.jstock.util;
import java.util.Map;
import java.net.URL;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;

/**
 * @author kaare
 *
 */
public class Util {
	static Logger logger = Logger.getLogger(Util.class);
	/**
	 * Creates a URL from a string and a hashmap with a substition table.
	 * '<' is used as start substitution and '>' for end substitution. Whatever is 
	 * between is used as key in the hashmap and replaced with what is returned.  
	 * 
	 * @param rawURL a URL that may includes fields that shall be replaced
	 * @param substituteMap a replacement HashMap 
	 * @return a presumably clean URL
	 */
	
	public static URL URLCreator(String rawURL, Map substituteMap ) {
		String leftLimiter = "<";
		char leftLimiterChar = '<';
		char rightLimiter = '>';
		StringBuffer resultURL = null;
		URL url = null;
		if (rawURL == null || substituteMap == null) {
			logger.error("Input URL string or HashMap is null");
			return null;
		}
		
		// Replace all '>' with '<' in order to split the string completly
		// All "keywords" will be in a separate line in the array.
		String tempURL = rawURL.replace(rightLimiter, leftLimiterChar);
		String [] splitRawURL = tempURL.split(leftLimiter);
		if (splitRawURL != null) {
			String replacementStr, str;
			resultURL = new StringBuffer(splitRawURL[0]);	
			for (int i = 1; i < splitRawURL.length; i ++) {
				str = splitRawURL[i];
				if (str != null) {
					replacementStr = (String) substituteMap.get(str);
					if (replacementStr != null) {
						// Replace from hashmap
						resultURL.append (replacementStr);
					} else {
						// Copy string
						resultURL.append (str);
					} 
					//System.out.println(replacementStr);
				} 
			}
		}
		try {
			url = new URL(resultURL.toString());	
		} catch (Exception e) {
			logger.error(" Unable to create URL" + e.toString());
			//logger.error(e.);
		}
		logger.debug("URL is " + url);
		return url;
	}
	
	/**
	 * Downloads the content of an URL and saves it to a file. It's up to the calling 
	 * class to ensure that the file exits or can be created. If the file exits the 
	 * previous content is overwritten. 
	 * 
	 * The method is quite long, but mostly due to exception handling.
	 * 
	 * @param url the URL to download
	 * @param file to save to
	 */
	public static void downloadURL(URL url, File file) {
		int bufferLength = 1024 * 100;
		InputStream inStream = null;
		OutputStream outStream = null;
		byte[] buffer = new byte[bufferLength];
		try {
			//Open input & output streams
			inStream = new BufferedInputStream(url.openStream());
			outStream = new FileOutputStream(file);
		} catch (FileNotFoundException notFoundE) {
			logger.error("Unable to create file "+ file.toString());
			logger.error(notFoundE.toString());
		} catch (IOException ioE) {
			logger.error("Unable to open URL "+ url.toString());
			logger.error(ioE.toString());			
		} catch (Exception e) {
			logger.error("Unable to get Url or save to file "+ e.toString());
		}
		// Start reading
		int bytesRead = 0;
		try {
			while (bytesRead >= 0) {
				bytesRead = inStream.read(buffer);
				outStream.write(buffer, 0, bytesRead);
				outStream.flush();
			} 
		}catch (FileNotFoundException notFoundE) {
				logger.error("Unable to write to "+ file.toString());
				logger.error(notFoundE.toString());
		} catch (IOException ioE) {
				logger.error("Unable to read from "+ url.toString());
				logger.error(ioE.toString());			
		}	
		// Closing streams
		if (inStream != null) {
			try {
				inStream.close();
			} catch (IOException ioE) {
				logger.error("Unable to close inputstream from "+ url.toString());
				logger.error(ioE.toString());
			}
		}
		if (outStream != null) {
			try {
				outStream.close();
			} catch (IOException ioE) {
				logger.error("Unable to close inputstream from "+ url.toString());
				logger.error(ioE.toString());
			}
		}
	}
}
